@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2016-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2016-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,22 +31,22 @@
  */
 
 #define _GNU_SOURCE
+#include "dlock.h"
+#include "pqos.h"
+#include "tsc.h"
+
+#include <assert.h>
+#include <ctype.h>
+#include <fcntl.h>
+#include <signal.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdarg.h>
 #include <string.h>
-#include <unistd.h>
-#include <ctype.h>
-#include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
+#include <sys/types.h>
 #include <time.h>
-#include <signal.h>
-#include <assert.h>
-
-#include <pqos.h>
-#include "dlock.h"
-#include "tsc.h"
+#include <unistd.h>
 
 #define DIM(x) (sizeof(x) / sizeof(x[0]))
 #define MB     (1024 * 1024)
@@ -81,9 +81,6 @@ init_memory(const size_t sz)
         char *p = NULL;
         size_t i;
 
-        if (sz <= 0)
-                return NULL;
-
         p = (char *)malloc(sz);
         if (p == NULL)
                 return NULL;
@@ -288,8 +285,7 @@ init_pqos(void)
         }
 
         /* Reset CAT */
-        ret = pqos_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                               PQOS_MBA_ANY);
+        ret = pqos_alloc_reset_config(NULL);
         if (ret != PQOS_RETVAL_OK) {
                 pqos_fini();
                 printf("Error resetting CAT!\n");
