@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,21 +38,18 @@
  * Module operate directly on CAT registers.
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <pthread.h>
-
-#include "pqos.h"
+#include "allocation.h"
 
 #include "cap.h"
-#include "allocation.h"
-#include "os_allocation.h"
-
-#include "machine.h"
-#include "types.h"
-#include "log.h"
 #include "cpu_registers.h"
 #include "cpuinfo.h"
+#include "log.h"
+#include "machine.h"
+#include "os_allocation.h"
+
+#include <pthread.h>
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * ---------------------------------------
@@ -136,15 +133,13 @@ hw_alloc_assoc_unused(const unsigned technology,
         int l2cat_id_set = l2_req;
         int l3cat_id_set = l3_req;
         int mba_id_set = mba_req;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         unsigned used_classes[PQOS_MAX_COS];
 
         if (class_id == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_cap_get(&cap, &cpu);
-
         memset(used_classes, 0, sizeof(used_classes));
 
         ret = pqos_l3ca_get_cos_num(cap, &num_l3_cos);
@@ -290,13 +285,25 @@ hw_l3ca_set(const unsigned l3cat_id,
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
         int cdp_enabled = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(ca != NULL);
         ASSERT(num_ca != 0);
 
-        _pqos_cap_get(&cap, &cpu);
+        /* Check L3 CBM is non-contiguous */
+        if (!cap_get_l3ca_non_contignous()) {
+                unsigned idx;
+                /* Check all COS CBM are contiguous */
+                for (idx = 0; idx < num_ca; idx++) {
+                        if (!IS_CONTIGNOUS(ca[idx])) {
+                                LOG_ERROR("L3 CAT COS%u bit mask is not "
+                                          "contiguous!\n",
+                                          ca[idx].class_id);
+                                return PQOS_RETVAL_PARAM;
+                        }
+                }
+        }
 
         ret = pqos_l3ca_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
@@ -369,15 +376,13 @@ hw_l3ca_get(const unsigned l3cat_id,
         uint64_t val = 0;
         int retval = MACHINE_RETVAL_OK;
         int cdp_enabled = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(num_ca != NULL);
         ASSERT(ca != NULL);
         ASSERT(max_num_ca != 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         ret = pqos_l3ca_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return ret; /**< perhaps no L3CA capability */
@@ -437,13 +442,11 @@ hw_l3ca_get_min_cbm_bits(unsigned *min_cbm_bits)
         int technology = 1 << PQOS_CAP_TYPE_L3CA;
         const struct pqos_capability *l3_cap = NULL;
         struct pqos_l3ca l3ca_config[PQOS_MAX_L3CA_COS];
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(min_cbm_bits != NULL);
 
-        _pqos_cap_get(&cap, &cpu);
-
         /**
          * Get L3 CAT capabilities
          */
@@ -563,13 +566,25 @@ hw_l2ca_set(const unsigned l2id,
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
         int cdp_enabled = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(ca != NULL);
         ASSERT(num_ca != 0);
 
-        _pqos_cap_get(&cap, &cpu);
+        /* Check L2 CBM is non-contiguous */
+        if (!cap_get_l2ca_non_contignous()) {
+                unsigned idx;
+                /* Check all COS CBM are contiguous */
+                for (idx = 0; idx < num_ca; idx++) {
+                        if (!IS_CONTIGNOUS(ca[idx])) {
+                                LOG_ERROR("L2 CAT COS%u bit mask is not "
+                                          "contiguous!\n",
+                                          ca[idx].class_id);
+                                return PQOS_RETVAL_PARAM;
+                        }
+                }
+        }
 
         /*
          * Check if L2 CAT is supported
@@ -654,15 +669,13 @@ hw_l2ca_get(const unsigned l2id,
         unsigned i = 0, count = 0;
         unsigned core = 0;
         int cdp_enabled = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(num_ca != NULL);
         ASSERT(ca != NULL);
         ASSERT(max_num_ca != 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         ret = pqos_l2ca_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
@@ -723,15 +736,13 @@ hw_l2ca_get_min_cbm_bits(unsigned *min_cbm_bits)
         unsigned *l2ids = NULL, l2id_num = 0, l2id;
         unsigned class_id, l2ca_num, ways, i;
         int technology = 1 << PQOS_CAP_TYPE_L2CA;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         const struct pqos_capability *l2_cap = NULL;
         struct pqos_l2ca l2ca_config[PQOS_MAX_L2CA_COS];
 
         ASSERT(min_cbm_bits != NULL);
 
-        _pqos_cap_get(&cap, &cpu);
-
         /**
          * Get L2 CAT capabilities
          */
@@ -849,14 +860,12 @@ hw_mba_set(const unsigned mba_id,
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0, step = 0;
         const struct pqos_capability *mba_cap = NULL;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(requested != NULL);
         ASSERT(num_cos != 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         /**
          * Check if MBA is supported
          */
@@ -936,15 +945,13 @@ hw_mba_set_amd(const unsigned mba_id,
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         const struct pqos_capability *mba_cap = NULL;
 
         ASSERT(requested != NULL);
         ASSERT(num_cos != 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         /**
          * Check if MBA is supported
          */
@@ -1011,15 +1018,13 @@ hw_mba_get(const unsigned mba_id,
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(num_cos != NULL);
         ASSERT(mba_tab != NULL);
         ASSERT(max_num_cos != 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         ret = pqos_mba_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return ret; /**< no MBA capability */
@@ -1056,15 +1061,13 @@ hw_mba_get_amd(const unsigned mba_id,
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i = 0, count = 0, core = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(num_cos != NULL);
         ASSERT(mba_tab != NULL);
         ASSERT(max_num_cos != 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         ret = pqos_mba_get_cos_num(cap, &count);
         if (ret != PQOS_RETVAL_OK)
                 return ret; /**< no MBA capability */
@@ -1119,10 +1122,8 @@ hw_alloc_assoc_set(const unsigned lcore, const unsigned class_id)
 {
         int ret = PQOS_RETVAL_OK;
         unsigned num_l2_cos = 0, num_l3_cos = 0, num_mba_cos = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
-
-        _pqos_cap_get(&cap, &cpu);
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ret = pqos_cpu_check_core(cpu, lcore);
         if (ret != PQOS_RETVAL_OK)
@@ -1157,10 +1158,8 @@ hw_alloc_assoc_get(const unsigned lcore, unsigned *class_id)
         const struct pqos_capability *l2_cap = NULL;
         const struct pqos_capability *mba_cap = NULL;
         int ret = PQOS_RETVAL_OK;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
-
-        _pqos_cap_get(&cap, &cpu);
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(class_id != NULL);
 
@@ -1201,15 +1200,13 @@ hw_alloc_assign(const unsigned technology,
         unsigned l3cat_id = 0, l2cat_id = 0, mba_id = 0;
         unsigned i;
         int ret;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(core_num > 0);
         ASSERT(core_array != NULL);
         ASSERT(class_id != NULL);
         ASSERT(technology != 0);
 
-        _pqos_cap_get(NULL, &cpu);
-
         /* Check if core belongs to one resource entity */
         for (i = 0; i < core_num; i++) {
                 const struct pqos_coreinfo *pi = NULL;
@@ -1284,12 +1281,10 @@ hw_alloc_reset_l3cdp(const unsigned l3cat_id_num,
                      const int enable)
 {
         unsigned j = 0;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(l3cat_id_num > 0 && l3cat_ids != NULL);
 
-        _pqos_cap_get(NULL, &cpu);
-
         LOG_INFO("%s L3 CDP across sockets...\n",
                  (enable) ? "Enabling" : "Disabling");
 
@@ -1326,12 +1321,10 @@ hw_alloc_reset_l2cdp(const unsigned l2id_num,
 {
         unsigned i = 0;
         int ret;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(l2id_num > 0 && l2ids != NULL);
 
-        _pqos_cap_get(NULL, &cpu);
-
         LOG_INFO("%s L2 CDP across clusters...\n",
                  (enable) ? "Enabling" : "Disabling");
 
@@ -1384,9 +1377,7 @@ hw_alloc_reset_assoc(void)
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i;
-        const struct pqos_cpuinfo *cpu;
-
-        _pqos_cap_get(NULL, &cpu);
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         for (i = 0; i < cpu->num_cores; i++)
                 if (hw_alloc_assoc_write(cpu->cores[i].lcore, 0) !=
@@ -1397,9 +1388,7 @@ hw_alloc_reset_assoc(void)
 }
 
 int
-hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-               const enum pqos_cdp_config l2_cdp_cfg,
-               const enum pqos_mba_config mba_cfg)
+hw_alloc_reset(const struct pqos_alloc_config *cfg)
 {
         unsigned *l3cat_ids = NULL;
         unsigned l3cat_id_num = 0;
@@ -1407,8 +1396,8 @@ hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
         unsigned mba_id_num = 0;
         unsigned *l2ids = NULL;
         unsigned l2id_num = 0;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         const struct pqos_capability *alloc_cap = NULL;
         const struct pqos_cap_l3ca *l3_cap = NULL;
         const struct pqos_cap_l2ca *l2_cap = NULL;
@@ -1419,21 +1408,29 @@ hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
         unsigned max_l2_cos = 0;
         unsigned j;
         int cdp_supported;
+        enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
+        enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
+        enum pqos_mba_config mba_cfg = PQOS_MBA_ANY;
 
-        ASSERT(l3_cdp_cfg == PQOS_REQUIRE_CDP_ON ||
-               l3_cdp_cfg == PQOS_REQUIRE_CDP_OFF ||
-               l3_cdp_cfg == PQOS_REQUIRE_CDP_ANY);
+        ASSERT(cfg == NULL || cfg->l3_cdp == PQOS_REQUIRE_CDP_ON ||
+               cfg->l3_cdp == PQOS_REQUIRE_CDP_OFF ||
+               cfg->l3_cdp == PQOS_REQUIRE_CDP_ANY);
 
-        ASSERT(l2_cdp_cfg == PQOS_REQUIRE_CDP_ON ||
-               l2_cdp_cfg == PQOS_REQUIRE_CDP_OFF ||
-               l2_cdp_cfg == PQOS_REQUIRE_CDP_ANY);
+        ASSERT(cfg == NULL || cfg->l2_cdp == PQOS_REQUIRE_CDP_ON ||
+               cfg->l2_cdp == PQOS_REQUIRE_CDP_OFF ||
+               cfg->l2_cdp == PQOS_REQUIRE_CDP_ANY);
 
-        ASSERT(mba_cfg == PQOS_MBA_DEFAULT || mba_cfg == PQOS_MBA_CTRL ||
-               mba_cfg == PQOS_MBA_ANY);
+        ASSERT(cfg == NULL || cfg->mba == PQOS_MBA_DEFAULT ||
+               cfg->mba == PQOS_MBA_CTRL || cfg->mba == PQOS_MBA_ANY);
 
-        _pqos_cap_get(&cap, &cpu);
         cpuinfo_get_config(&vconfig);
 
+        if (cfg != NULL) {
+                l3_cdp_cfg = cfg->l3_cdp;
+                l2_cdp_cfg = cfg->l2_cdp;
+                mba_cfg = cfg->mba;
+        }
+
         /* Get L3 CAT capabilities */
         (void)pqos_cap_get_type(cap, PQOS_CAP_TYPE_L3CA, &alloc_cap);
         if (alloc_cap != NULL)
@@ -1676,3 +1673,18 @@ pqos_alloc_reset_exit:
                 free(l2ids);
         return ret;
 }
+
+int
+alloc_is_bitmask_contiguous(uint64_t bitmask)
+{
+        if (bitmask == 0)
+                return 0;
+
+        while ((bitmask & 1) == 0) /**< Shift until 1 found at position 0 */
+                bitmask >>= 1;
+
+        while ((bitmask & 1) != 0) /**< Shift until 0 found at position 0 */
+                bitmask >>= 1;
+
+        return (bitmask) ? 0 : 1; /**< non-zero bitmask is not contiguous */
+}
