@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,8 +42,21 @@
 extern "C" {
 #endif
 
-#include "types.h"
 #include "pqos.h"
+#include "types.h"
+
+#define IS_CONTIGNOUS(ca)                                                      \
+        ({                                                                     \
+                int ret;                                                       \
+                                                                               \
+                if (ca.cdp)                                                    \
+                        ret = alloc_is_bitmask_contiguous(ca.u.s.data_mask) && \
+                              alloc_is_bitmask_contiguous(ca.u.s.code_mask);   \
+                else                                                           \
+                        ret = alloc_is_bitmask_contiguous(ca.u.ways_mask);     \
+                                                                               \
+                ret;                                                           \
+        })
 
 /**
  * Types of possible PQoS allocation technologies
@@ -193,22 +206,18 @@ PQOS_LOCAL int hw_alloc_release(const unsigned *core_array,
  * As part of allocation reset CDP reconfiguration can be performed.
  * This can be requested via \a l3_cdp_cfg or \a l2_cdp_cfg.
  *
- * @param [in] l3_cdp_cfg requested L3 CAT CDP config
- * @param [in] l2_cdp_cfg requested L2 CAT CDP config
- * @param [in] mba_cfg requested MBA CTRL config
+ * @param [in] cfg requested configuration
  *
  * @return Operation status
  * @retval PQOS_RETVAL_OK on success
  */
-PQOS_LOCAL int hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-                              const enum pqos_cdp_config l2_cdp_cfg,
-                              const enum pqos_mba_config mba_cfg);
+PQOS_LOCAL int hw_alloc_reset(const struct pqos_alloc_config *cfg);
 
 /**
  * @brief Enables or disables L3 CDP across selected CPU sockets
  *
- * @param [in] sockets_num dimension of \a sockets array
- * @param [in] sockets array with socket ids to change CDP config on
+ * @param [in] l3cat_id_num dimension of \a sockets array
+ * @param [in] l3cat_ids array with socket ids to change CDP config on
  * @param [in] enable CDP enable/disable flag, 1 - enable, 0 - disable
  *
  * @return Operations status
@@ -376,7 +385,7 @@ PQOS_LOCAL int hw_mba_set(const unsigned mba_id,
 /**
  * @brief Hardware interface to set classes of service
  *        defined by \a requested on \a mba_id
- * @NOTE: This function is specific to AMD
+ * @note: This function is specific to AMD
  *
  * @param [in]  mba_id
  * @param [in]  num_cos number of classes of service at \a ca
@@ -410,7 +419,7 @@ PQOS_LOCAL int hw_mba_get(const unsigned mba_id,
 
 /**
  * @brief Hardware interface to read MBA from \a mba_id
- * @NOTE: This function is specific to AMD
+ * @note: This function is specific to AMD
  *
  * @param [in]  mba_id MBA resource id
  * @param [in]  max_num_cos maximum number of classes of service
@@ -426,6 +435,22 @@ PQOS_LOCAL int hw_mba_get_amd(const unsigned mba_id,
                               unsigned *num_cos,
                               struct pqos_mba *mba_tab);
 
+/**
+ * @brief Tests if \a bitmask is contiguous
+ *
+ * Zero bit mask is regarded as not contiguous.
+ *
+ * The function shifts out first group of contiguous 1's in the bit mask.
+ * Next it checks remaining bitmask content to make a decision.
+ *
+ * @param bitmask bit mask to be validated for contiguity
+ *
+ * @return Bit mask contiguity check result
+ * @retval 0 not contiguous
+ * @retval 1 contiguous
+ */
+PQOS_LOCAL int alloc_is_bitmask_contiguous(uint64_t bitmask);
+
 #ifdef __cplusplus
 }
 #endif
