@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -37,10 +37,12 @@
 #include "cap.h"
 #include "cpuinfo.h"
 #include "hw_monitoring.h"
+#include "lock.h"
 #include "log.h"
 #include "monitoring.h"
 #include "os_allocation.h"
 #include "os_monitoring.h"
+#include "pqos_internal.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -61,11 +63,12 @@ static struct pqos_api {
         /** Reads RMID association lcore */
         int (*mon_assoc_get)(const unsigned lcore, pqos_rmid_t *rmid);
         /** Starts resource monitoring on selected group of cores */
-        int (*mon_start)(const unsigned num_cores,
-                         const unsigned *cores,
-                         const enum pqos_mon_event event,
-                         void *context,
-                         struct pqos_mon_data *group);
+        int (*mon_start_cores)(const unsigned num_cores,
+                               const unsigned *cores,
+                               const enum pqos_mon_event event,
+                               void *context,
+                               struct pqos_mon_data *group,
+                               const struct pqos_mon_options *opt);
         /** Starts resource monitoring of selected pids */
         int (*mon_start_pids)(const unsigned num_pids,
                               const pid_t *pids,
@@ -114,9 +117,7 @@ static struct pqos_api {
         int (*alloc_release_pid)(const pid_t *task_array,
                                  const unsigned task_num);
         /** Resets configuration of allocation technologies */
-        int (*alloc_reset)(const enum pqos_cdp_config l3_cdp_cfg,
-                           const enum pqos_cdp_config l2_cdp_cfg,
-                           const enum pqos_mba_config mba_cfg);
+        int (*alloc_reset)(const struct pqos_alloc_config *cfg);
 
         /** Sets L3 classes of service */
         int (*l3ca_set)(const unsigned l3cat_id,
@@ -176,7 +177,7 @@ api_init(int interface, enum pqos_vendor vendor)
         if (interface == PQOS_INTER_MSR) {
                 api.mon_reset = hw_mon_reset;
                 api.mon_assoc_get = hw_mon_assoc_get;
-                api.mon_start = hw_mon_start;
+                api.mon_start_cores = hw_mon_start_cores;
                 api.mon_stop = hw_mon_stop;
                 api.alloc_assoc_set = hw_alloc_assoc_set;
                 api.alloc_assoc_get = hw_alloc_assoc_get;
@@ -202,7 +203,7 @@ api_init(int interface, enum pqos_vendor vendor)
         } else if (interface == PQOS_INTER_OS ||
                    interface == PQOS_INTER_OS_RESCTRL_MON) {
                 api.mon_reset = os_mon_reset;
-                api.mon_start = os_mon_start;
+                api.mon_start_cores = os_mon_start_cores;
                 api.mon_start_pids = os_mon_start_pids;
                 api.mon_add_pids = os_mon_add_pids;
                 api.mon_remove_pids = os_mon_remove_pids;
@@ -242,7 +243,7 @@ api_init(int interface, enum pqos_vendor vendor)
         ({                                                                     \
                 int ret;                                                       \
                                                                                \
-                _pqos_api_lock();                                              \
+                lock_get();                                                    \
                 do {                                                           \
                         ret = _pqos_check_init(1);                             \
                         if (ret != PQOS_RETVAL_OK)                             \
@@ -255,7 +256,7 @@ api_init(int interface, enum pqos_vendor vendor)
                                 ret = PQOS_RETVAL_RESOURCE;                    \
                         }                                                      \
                 } while (0);                                                   \
-                _pqos_api_unlock();                                            \
+                lock_release();                                                \
                                                                                \
                 ret;                                                           \
         })
@@ -349,30 +350,50 @@ pqos_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                  const enum pqos_cdp_config l2_cdp_cfg,
                  const enum pqos_mba_config mba_cfg)
 {
-        if (l3_cdp_cfg != PQOS_REQUIRE_CDP_ON &&
-            l3_cdp_cfg != PQOS_REQUIRE_CDP_OFF &&
-            l3_cdp_cfg != PQOS_REQUIRE_CDP_ANY) {
-                LOG_ERROR("Unrecognized L3 CDP configuration setting %d!\n",
-                          l3_cdp_cfg);
-                return PQOS_RETVAL_PARAM;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
+
+        return pqos_alloc_reset_config(&cfg);
+}
+
+int
+pqos_alloc_reset_config(const struct pqos_alloc_config *cfg)
+{
+        /* validate parameters */
+        if (cfg != NULL) {
+                if (cfg->l3_cdp != PQOS_REQUIRE_CDP_ON &&
+                    cfg->l3_cdp != PQOS_REQUIRE_CDP_OFF &&
+                    cfg->l3_cdp != PQOS_REQUIRE_CDP_ANY) {
+                        LOG_ERROR(
+                            "Unrecognized L3 CDP configuration setting %d!\n",
+                            cfg->l3_cdp);
+                        return PQOS_RETVAL_PARAM;
+                }
+
+                if (cfg->l2_cdp != PQOS_REQUIRE_CDP_ON &&
+                    cfg->l2_cdp != PQOS_REQUIRE_CDP_OFF &&
+                    cfg->l2_cdp != PQOS_REQUIRE_CDP_ANY) {
+                        LOG_ERROR(
+                            "Unrecognized L2 CDP configuration setting %d!\n",
+                            cfg->l2_cdp);
+                        return PQOS_RETVAL_PARAM;
+                }
+
+                if (cfg->mba != PQOS_MBA_ANY && cfg->mba != PQOS_MBA_DEFAULT &&
+                    cfg->mba != PQOS_MBA_CTRL) {
+                        LOG_ERROR(
+                            "Unrecognized MBA configuration setting %d!\n",
+                            cfg->mba);
+                        return PQOS_RETVAL_PARAM;
+                }
         }
 
-        if (l2_cdp_cfg != PQOS_REQUIRE_CDP_ON &&
-            l2_cdp_cfg != PQOS_REQUIRE_CDP_OFF &&
-            l2_cdp_cfg != PQOS_REQUIRE_CDP_ANY) {
-                LOG_ERROR("Unrecognized L2 CDP configuration setting %d!\n",
-                          l2_cdp_cfg);
-                return PQOS_RETVAL_PARAM;
-        }
-
-        if (mba_cfg != PQOS_MBA_ANY && mba_cfg != PQOS_MBA_DEFAULT &&
-            mba_cfg != PQOS_MBA_CTRL) {
-                LOG_ERROR("Unrecognized MBA configuration setting %d!\n",
-                          mba_cfg);
-                return PQOS_RETVAL_PARAM;
-        }
-
-        return API_CALL(alloc_reset, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        return API_CALL(alloc_reset, cfg);
 }
 
 unsigned *
@@ -384,11 +405,11 @@ pqos_pid_get_pid_assoc(const unsigned class_id, unsigned *count)
         if (count == NULL)
                 return NULL;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return NULL;
         }
 
@@ -399,7 +420,7 @@ pqos_pid_get_pid_assoc(const unsigned class_id, unsigned *count)
         } else
                 LOG_INFO(UNSUPPORTED_INTERFACE);
 
-        _pqos_api_unlock();
+        lock_release();
 
         return tasks;
 }
@@ -410,35 +431,6 @@ pqos_pid_get_pid_assoc(const unsigned class_id, unsigned *count)
  * =======================================
  */
 
-/**
- * @brief Tests if \a bitmask is contiguous
- *
- * Zero bit mask is regarded as not contiguous.
- *
- * The function shifts out first group of contiguous 1's in the bit mask.
- * Next it checks remaining bitmask content to make a decision.
- *
- * @param bitmask bit mask to be validated for contiguity
- *
- * @return Bit mask contiguity check result
- * @retval 0 not contiguous
- * @retval 1 contiguous
- */
-static int
-is_contiguous(uint64_t bitmask)
-{
-        if (bitmask == 0)
-                return 0;
-
-        while ((bitmask & 1) == 0) /**< Shift until 1 found at position 0 */
-                bitmask >>= 1;
-
-        while ((bitmask & 1) != 0) /**< Shift until 0 found at position 0 */
-                bitmask >>= 1;
-
-        return (bitmask) ? 0 : 1; /**< non-zero bitmask is not contiguous */
-}
-
 int
 pqos_l3ca_set(const unsigned l3cat_id,
               const unsigned num_cos,
@@ -450,20 +442,19 @@ pqos_l3ca_set(const unsigned l3cat_id,
                 return PQOS_RETVAL_PARAM;
 
         /**
-         * Check if class bitmasks are contiguous.
+         * Check if class bitmasks are zero.
          */
         for (i = 0; i < num_cos; i++) {
-                int is_contig = 0;
+                int is_non_zero = 0;
 
-                if (ca[i].cdp) {
-                        is_contig = is_contiguous(ca[i].u.s.data_mask) &&
-                                    is_contiguous(ca[i].u.s.code_mask);
-                } else
-                        is_contig = is_contiguous(ca[i].u.ways_mask);
+                if (ca[i].cdp)
+                        is_non_zero =
+                            ca[i].u.s.data_mask && ca[i].u.s.code_mask;
+                else
+                        is_non_zero = ca[i].u.ways_mask;
 
-                if (!is_contig) {
-                        LOG_ERROR("L3 COS%u bit mask is not contiguous!\n",
-                                  ca[i].class_id);
+                if (!is_non_zero) {
+                        LOG_ERROR("L3 COS%u bit mask is 0!\n", ca[i].class_id);
                         return PQOS_RETVAL_PARAM;
                 }
         }
@@ -509,20 +500,19 @@ pqos_l2ca_set(const unsigned l2id,
                 return PQOS_RETVAL_PARAM;
 
         /**
-         * Check if class bitmasks are contiguous
+         * Check if class bitmasks are zero.
          */
         for (i = 0; i < num_cos; i++) {
-                int is_contig = 0;
+                int is_non_zero = 0;
 
-                if (ca[i].cdp) {
-                        is_contig = is_contiguous(ca[i].u.s.data_mask) &&
-                                    is_contiguous(ca[i].u.s.code_mask);
-                } else
-                        is_contig = is_contiguous(ca[i].u.ways_mask);
+                if (ca[i].cdp)
+                        is_non_zero =
+                            ca[i].u.s.data_mask && ca[i].u.s.code_mask;
+                else
+                        is_non_zero = ca[i].u.ways_mask;
 
-                if (!is_contig) {
-                        LOG_ERROR("L2 COS%u bit mask is not contiguous!\n",
-                                  ca[i].class_id);
+                if (!is_non_zero) {
+                        LOG_ERROR("L2 COS%u bit mask is 0!\n", ca[i].class_id);
                         return PQOS_RETVAL_PARAM;
                 }
         }
@@ -569,11 +559,11 @@ pqos_mba_set(const unsigned mba_id,
         if (requested == NULL || num_cos == 0)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
@@ -589,7 +579,7 @@ pqos_mba_set(const unsigned mba_id,
                      requested[i].mb_max > vconfig->mba_max)) {
                         LOG_ERROR("MBA COS%u rate out of range (from 1-%d)!\n",
                                   requested[i].class_id, vconfig->mba_max);
-                        _pqos_api_unlock();
+                        lock_release();
                         return PQOS_RETVAL_PARAM;
                 }
         }
@@ -601,7 +591,7 @@ pqos_mba_set(const unsigned mba_id,
                 ret = PQOS_RETVAL_RESOURCE;
         }
 
-        _pqos_api_unlock();
+        lock_release();
 
         return ret;
 }
@@ -647,6 +637,7 @@ pqos_mon_start(const unsigned num_cores,
                struct pqos_mon_data *group)
 {
         int ret;
+        struct pqos_mon_options opt;
 
         if (group == NULL || cores == NULL || num_cores == 0 || event == 0)
                 return PQOS_RETVAL_PARAM;
@@ -673,37 +664,107 @@ pqos_mon_start(const unsigned num_cores,
                 return PQOS_RETVAL_PARAM;
         }
 
-        _pqos_api_lock();
+        struct pqos_mon_data_internal *mem = malloc(sizeof(*group->intl));
+
+        if (mem == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
+                free(mem);
                 return ret;
         }
 
         memset(group, 0, sizeof(*group));
-        group->intl =
-            (struct pqos_mon_data_internal *)malloc(sizeof(*group->intl));
-        if (group->intl == NULL) {
-                ret = PQOS_RETVAL_RESOURCE;
-                goto pqos_mon_start_exit;
-        }
+        group->intl = mem;
         memset(group->intl, 0, sizeof(*group->intl));
 
-        if (api.mon_start != NULL)
-                ret = api.mon_start(num_cores, cores, event, context, group);
+        memset(&opt, 0, sizeof(opt));
+        if (api.mon_start_cores != NULL)
+                ret = api.mon_start_cores(num_cores, cores, event, context,
+                                          group, &opt);
         else {
                 LOG_INFO(UNSUPPORTED_INTERFACE);
                 ret = PQOS_RETVAL_RESOURCE;
         }
 
-pqos_mon_start_exit:
         if (ret == PQOS_RETVAL_OK)
                 group->valid = GROUP_VALID_MARKER;
-        else if (group->intl != NULL)
-                free(group->intl);
+        else if (mem != NULL)
+                free(mem);
 
-        _pqos_api_unlock();
+        lock_release();
+
+        return ret;
+}
+
+int
+pqos_mon_start_cores(const unsigned num_cores,
+                     const unsigned *cores,
+                     const enum pqos_mon_event event,
+                     void *context,
+                     struct pqos_mon_data **group)
+{
+        struct pqos_mon_options opt;
+
+        memset(&opt, 0, sizeof(opt));
+
+        return pqos_mon_start_cores_ext(num_cores, cores, event, context, group,
+                                        &opt);
+}
+
+int
+pqos_mon_start_cores_ext(const unsigned num_cores,
+                         const unsigned *cores,
+                         const enum pqos_mon_event event,
+                         void *context,
+                         struct pqos_mon_data **group,
+                         const struct pqos_mon_options *opt)
+{
+        int ret;
+        struct pqos_mon_data *data = NULL;
+
+        if (group == NULL || cores == NULL || num_cores == 0 || event == 0 ||
+            opt == NULL)
+                return PQOS_RETVAL_PARAM;
+
+        /**
+         * Validate event parameter
+         * - only combinations of events allowed
+         * - do not allow non-PQoS events to be monitored on its own
+         */
+        if (event & (~(PQOS_MON_EVENT_L3_OCCUP | PQOS_MON_EVENT_LMEM_BW |
+                       PQOS_MON_EVENT_TMEM_BW | PQOS_MON_EVENT_RMEM_BW |
+                       PQOS_PERF_EVENT_IPC | PQOS_PERF_EVENT_LLC_MISS |
+                       PQOS_PERF_EVENT_LLC_REF)))
+                return PQOS_RETVAL_PARAM;
+
+        if ((event & (PQOS_MON_EVENT_L3_OCCUP | PQOS_MON_EVENT_LMEM_BW |
+                      PQOS_MON_EVENT_TMEM_BW | PQOS_MON_EVENT_RMEM_BW)) == 0 &&
+            (event & (PQOS_PERF_EVENT_IPC | PQOS_PERF_EVENT_LLC_MISS |
+                      PQOS_PERF_EVENT_LLC_REF)) != 0) {
+                LOG_ERROR("Only PMU events selected for monitoring\n");
+                return PQOS_RETVAL_PARAM;
+        }
+
+        data = calloc(1, sizeof(*data) + sizeof(struct pqos_mon_data_internal));
+        if (data == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        data->intl = (struct pqos_mon_data_internal *)(&data[1]);
+        data->intl->manage_memory = 1;
+
+        ret = API_CALL(mon_start_cores, num_cores, cores, event, context, data,
+                       opt);
+
+        if (ret == PQOS_RETVAL_OK) {
+                data->valid = GROUP_VALID_MARKER;
+                *group = data;
+        } else if (data != NULL)
+                free(data);
 
         return ret;
 }
@@ -712,7 +773,6 @@ int
 pqos_mon_stop(struct pqos_mon_data *group)
 {
         int ret;
-        int manage_memory;
 
         if (group == NULL)
                 return PQOS_RETVAL_PARAM;
@@ -720,11 +780,11 @@ pqos_mon_stop(struct pqos_mon_data *group)
         if (group->valid != GROUP_VALID_MARKER)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
@@ -735,14 +795,14 @@ pqos_mon_stop(struct pqos_mon_data *group)
                 ret = PQOS_RETVAL_RESOURCE;
         }
 
-        manage_memory = group->intl->manage_memory;
-        free(group->intl);
-        if (manage_memory)
+        if (group->intl->manage_memory)
                 free(group);
-        else
+        else {
+                free(group->intl);
                 memset(group, 0, sizeof(*group));
+        }
 
-        _pqos_api_unlock();
+        lock_release();
 
         return ret;
 }
@@ -765,11 +825,11 @@ pqos_mon_poll(struct pqos_mon_data **groups, const unsigned num_groups)
                         return PQOS_RETVAL_PARAM;
         }
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
@@ -782,7 +842,7 @@ pqos_mon_poll(struct pqos_mon_data **groups, const unsigned num_groups)
                         ret = retval;
                 }
         }
-        _pqos_api_unlock();
+        lock_release();
 
         return ret;
 }
@@ -830,20 +890,22 @@ pqos_mon_start_pids(const unsigned num_pids,
                 return PQOS_RETVAL_PARAM;
         }
 
-        _pqos_api_lock();
+        struct pqos_mon_data_internal *mem = malloc(sizeof(*group->intl));
+
+        if (mem == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
+                free(mem);
                 return ret;
         }
 
         memset(group, 0, sizeof(*group));
-        group->intl = malloc(sizeof(*group->intl));
-        if (group->intl == NULL) {
-                ret = PQOS_RETVAL_RESOURCE;
-                goto pqos_mon_start_pids_exit;
-        }
+        group->intl = mem;
         memset(group->intl, 0, sizeof(*group->intl));
 
         if (api.mon_start_pids != NULL)
@@ -853,13 +915,62 @@ pqos_mon_start_pids(const unsigned num_pids,
                 ret = PQOS_RETVAL_RESOURCE;
         }
 
-pqos_mon_start_pids_exit:
         if (ret == PQOS_RETVAL_OK)
                 group->valid = GROUP_VALID_MARKER;
-        else if (group->intl != NULL)
-                free(group->intl);
+        else if (mem != NULL)
+                free(mem);
 
-        _pqos_api_unlock();
+        lock_release();
+
+        return ret;
+}
+
+int
+pqos_mon_start_pids2(const unsigned num_pids,
+                     const pid_t *pids,
+                     const enum pqos_mon_event event,
+                     void *context,
+                     struct pqos_mon_data **group)
+{
+        int ret;
+        struct pqos_mon_data *data = NULL;
+
+        if (group == NULL || num_pids == 0 || pids == NULL || event == 0)
+                return PQOS_RETVAL_PARAM;
+
+        /**
+         * Validate event parameter
+         * - only combinations of events allowed
+         * - do not allow non-PQoS events to be monitored on its own
+         */
+        if (event & (~(PQOS_MON_EVENT_L3_OCCUP | PQOS_MON_EVENT_LMEM_BW |
+                       PQOS_MON_EVENT_TMEM_BW | PQOS_MON_EVENT_RMEM_BW |
+                       PQOS_PERF_EVENT_IPC | PQOS_PERF_EVENT_LLC_MISS |
+                       PQOS_PERF_EVENT_LLC_REF)))
+                return PQOS_RETVAL_PARAM;
+
+        if ((event & (PQOS_MON_EVENT_L3_OCCUP | PQOS_MON_EVENT_LMEM_BW |
+                      PQOS_MON_EVENT_TMEM_BW | PQOS_MON_EVENT_RMEM_BW)) == 0 &&
+            (event & (PQOS_PERF_EVENT_IPC | PQOS_PERF_EVENT_LLC_MISS |
+                      PQOS_PERF_EVENT_LLC_REF)) != 0) {
+                LOG_ERROR("Only PMU events selected for monitoring\n");
+                return PQOS_RETVAL_PARAM;
+        }
+
+        data = calloc(1, sizeof(*data) + sizeof(struct pqos_mon_data_internal));
+        if (data == NULL)
+                return PQOS_RETVAL_RESOURCE;
+
+        data->intl = (struct pqos_mon_data_internal *)(&data[1]);
+        data->intl->manage_memory = 1;
+
+        ret = API_CALL(mon_start_pids, num_pids, pids, event, context, data);
+
+        if (ret == PQOS_RETVAL_OK) {
+                data->valid = GROUP_VALID_MARKER;
+                *group = data;
+        } else if (data != NULL)
+                free(data);
 
         return ret;
 }
@@ -914,45 +1025,20 @@ pqos_mon_start_uncore(const unsigned num_sockets,
                       PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE)) == 0)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
-
-        ret = _pqos_check_init(1);
-        if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
-                return ret;
-        }
-
-        data = calloc(1, sizeof(*data));
-        if (data == NULL) {
-                ret = PQOS_RETVAL_RESOURCE;
-                goto pqos_mon_start_uncore_exit;
-        }
-
-        data->intl = calloc(1, sizeof(*data->intl));
-        if (data->intl == NULL) {
-                ret = PQOS_RETVAL_RESOURCE;
-                goto pqos_mon_start_uncore_exit;
-        }
+        data = calloc(1, sizeof(*data) + sizeof(struct pqos_mon_data_internal));
+        if (data == NULL)
+                return PQOS_RETVAL_RESOURCE;
+        data->intl = (struct pqos_mon_data_internal *)(&data[1]);
         data->intl->manage_memory = 1;
 
-        if (api.mon_start_uncore != NULL)
-                ret = api.mon_start_uncore(num_sockets, sockets, event, context,
-                                           data);
-        else {
-                LOG_INFO(UNSUPPORTED_INTERFACE);
-                ret = PQOS_RETVAL_RESOURCE;
-        }
+        ret = API_CALL(mon_start_uncore, num_sockets, sockets, event, context,
+                       data);
 
-pqos_mon_start_uncore_exit:
         if (ret == PQOS_RETVAL_OK) {
                 data->valid = GROUP_VALID_MARKER;
                 *group = data;
-        } else if (data != NULL) {
-                free(data->intl);
+        } else if (data != NULL)
                 free(data);
-        }
-
-        _pqos_api_unlock();
 
         return ret;
 }
@@ -982,11 +1068,11 @@ pqos_mon_get_value(const struct pqos_mon_data *const group,
         if ((group->event & event_id) == 0)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
@@ -1051,7 +1137,7 @@ pqos_mon_get_value(const struct pqos_mon_data *const group,
                         *delta = _delta;
         }
 
-        _pqos_api_unlock();
+        lock_release();
 
         return ret;
 }
@@ -1070,17 +1156,17 @@ pqos_mon_get_ipc(const struct pqos_mon_data *const group, double *value)
         if ((group->event & PQOS_PERF_EVENT_IPC) == 0)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
         *value = group->values.ipc;
 
-        _pqos_api_unlock();
+        lock_release();
 
         return ret;
 }
