@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -39,8 +39,6 @@
  * Management functions include:
  * - management includes initializing and shutting down all other sub-modules
  *   including: monitoring, allocation, log, cpuinfo and machine
- * - provide functions for safe access to PQoS API - this is required for
- *   allocation and monitoring modules which also implement PQoS API
  *
  * Capability functions:
  * - monitoring detection, this is to discover all monitoring event types.
@@ -50,26 +48,24 @@
  *   technology.
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <fcntl.h>    /* O_CREAT */
-#include <unistd.h>   /* usleep(), lockf() */
-#include <sys/stat.h> /* S_Ixxx */
-#include <pthread.h>
-
 #include "cap.h"
-#include "os_cap.h"
-#include "hw_cap.h"
+
 #include "allocation.h"
-#include "monitoring.h"
-#include "cpuinfo.h"
-#include "machine.h"
-#include "types.h"
-#include "log.h"
 #include "api.h"
-#include "utils.h"
+#include "cpu_registers.h"
+#include "cpuinfo.h"
+#include "hw_cap.h"
+#include "lock.h"
+#include "log.h"
+#include "machine.h"
+#include "monitoring.h"
+#include "os_cap.h"
 #include "resctrl.h"
 #include "resctrl_alloc.h"
+#include "utils.h"
+
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * ---------------------------------------
@@ -77,15 +73,6 @@
  * ---------------------------------------
  */
 
-#ifndef LOCKFILE
-#ifdef __linux__
-#define LOCKFILE "/var/lock/libpqos"
-#endif
-#ifdef __FreeBSD__
-#define LOCKFILE "/var/tmp/libpqos.lockfile"
-#endif
-#endif /*!LOCKFILE*/
-
 #define PROC_CPUINFO "/proc/cpuinfo"
 
 /**
@@ -118,12 +105,6 @@ static const struct pqos_cpuinfo *m_cpu = NULL;
  */
 static int m_init_done = 0;
 
-/**
- * API thread/process safe access is secured through these locks.
- */
-static int m_apilock = -1;
-static pthread_mutex_t m_apilock_mutex;
-
 /**
  * Interface status
  *   0  PQOS_INTER_MSR
@@ -131,93 +112,23 @@ static pthread_mutex_t m_apilock_mutex;
  *   2  PQOS_INTER_OS_RESCTRL_MON
  */
 static enum pqos_interface m_interface = PQOS_INTER_MSR;
+
 /**
  * ---------------------------------------
- * Functions for safe multi-threading
+ * Local functions
  * ---------------------------------------
  */
 
-/**
- * @brief Initializes API locks
- *
- * @return Operation status
- * @retval 0 success
- * @retval -1 error
- */
-static int
-_pqos_api_init(void)
+enum pqos_interface
+_pqos_get_inter(void)
 {
-
-        const char *lock_filename = LOCKFILE;
-
-        if (m_apilock != -1)
-                return -1;
-
-        m_apilock = open(lock_filename, O_WRONLY | O_CREAT,
-                         S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
-        if (m_apilock == -1)
-                return -1;
-
-        if (pthread_mutex_init(&m_apilock_mutex, NULL) != 0) {
-                close(m_apilock);
-                m_apilock = -1;
-                return -1;
-        }
-
-        return 0;
+        return m_interface;
 }
 
-/**
- * @brief Uninitializes API locks
- *
- * @return Operation status
- * @retval 0 success
- * @retval -1 error
- */
-static int
-_pqos_api_exit(void)
+PQOS_STATIC void
+_pqos_set_inter(const enum pqos_interface iface)
 {
-        int ret = 0;
-
-        if (close(m_apilock) != 0)
-                ret = -1;
-
-        if (pthread_mutex_destroy(&m_apilock_mutex) != 0)
-                ret = -1;
-
-        m_apilock = -1;
-
-        return ret;
-}
-
-void
-_pqos_api_lock(void)
-{
-        int err = 0;
-
-        if (lockf(m_apilock, F_LOCK, 0) != 0)
-                err = 1;
-
-        if (pthread_mutex_lock(&m_apilock_mutex) != 0)
-                err = 1;
-
-        if (err)
-                LOG_ERROR("API lock error!\n");
-}
-
-void
-_pqos_api_unlock(void)
-{
-        int err = 0;
-
-        if (lockf(m_apilock, F_ULOCK, 0) != 0)
-                err = 1;
-
-        if (pthread_mutex_unlock(&m_apilock_mutex) != 0)
-                err = 1;
-
-        if (err)
-                LOG_ERROR("API unlock error!\n");
+        m_interface = iface;
 }
 
 /**
@@ -252,7 +163,7 @@ _pqos_check_init(const int expect)
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
                   const struct pqos_cpuinfo *cpu,
                   const enum pqos_interface iface)
@@ -260,6 +171,9 @@ cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
         struct pqos_cap_l3ca *cap = NULL;
         int ret;
 
+        if (r_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
         cap = (struct pqos_cap_l3ca *)malloc(sizeof(*cap));
         if (cap == NULL)
                 return PQOS_RETVAL_RESOURCE;
@@ -297,7 +211,7 @@ cap_l3ca_discover(struct pqos_cap_l3ca **r_cap,
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
                   const struct pqos_cpuinfo *cpu,
                   const enum pqos_interface iface)
@@ -305,6 +219,9 @@ cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
         struct pqos_cap_l2ca *cap = NULL;
         int ret;
 
+        if (r_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
         cap = (struct pqos_cap_l2ca *)malloc(sizeof(*cap));
         if (cap == NULL)
                 return PQOS_RETVAL_RESOURCE;
@@ -342,7 +259,7 @@ cap_l2ca_discover(struct pqos_cap_l2ca **r_cap,
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 cap_mba_discover(struct pqos_cap_mba **r_cap,
                  const struct pqos_cpuinfo *cpu,
                  const enum pqos_interface iface)
@@ -350,6 +267,9 @@ cap_mba_discover(struct pqos_cap_mba **r_cap,
         struct pqos_cap_mba *cap = NULL;
         int ret;
 
+        if (r_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
+
         cap = (struct pqos_cap_mba *)malloc(sizeof(*cap));
         if (cap == NULL)
                 return PQOS_RETVAL_RESOURCE;
@@ -390,7 +310,7 @@ cap_mba_discover(struct pqos_cap_mba **r_cap,
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 discover_capabilities(struct pqos_cap **p_cap,
                       const struct pqos_cpuinfo *cpu,
                       enum pqos_interface inter)
@@ -403,6 +323,8 @@ discover_capabilities(struct pqos_cap **p_cap,
         unsigned sz = 0;
         int ret = PQOS_RETVAL_RESOURCE;
 
+        if (p_cap == NULL || cpu == NULL)
+                return PQOS_RETVAL_PARAM;
         /**
          * Monitoring init
          */
@@ -586,8 +508,8 @@ error_exit:
  *
  * @return converted enumeration value into string
  */
-static const char *
-interface_to_string(enum pqos_interface interface)
+PQOS_STATIC const char *
+_cap_interface_to_string(enum pqos_interface interface)
 {
         switch (interface) {
         case PQOS_INTER_MSR:
@@ -612,15 +534,17 @@ interface_to_string(enum pqos_interface interface)
  * @return Operation status
  * @retval PQOS_RETVAL_OK success
  */
-static int
+PQOS_STATIC int
 discover_interface(enum pqos_interface requested_interface,
                    enum pqos_interface *interface)
 {
         char *environment = NULL;
 
         LOG_INFO("Requested interface: %s\n",
-                 interface_to_string(requested_interface));
+                 _cap_interface_to_string(requested_interface));
 
+        if (interface == NULL)
+                return PQOS_RETVAL_PARAM;
 #ifdef __linux__
         if (requested_interface != PQOS_INTER_MSR &&
             requested_interface != PQOS_INTER_OS &&
@@ -659,21 +583,21 @@ discover_interface(enum pqos_interface requested_interface,
                                   "Invalid interface enforcement selection.\n");
                         return PQOS_RETVAL_ERROR;
                 }
-        } else {
+        } else if (requested_interface == PQOS_INTER_AUTO) {
 #ifdef __linux__
-                if (requested_interface == PQOS_INTER_AUTO &&
-                    resctrl_is_supported() == PQOS_RETVAL_OK)
+                if (resctrl_is_supported() == PQOS_RETVAL_OK)
                         *interface = PQOS_INTER_OS;
-#else
-                if (requested_interface == PQOS_INTER_AUTO)
-                        *interface = PQOS_INTER_MSR;
-
-#endif
                 else
-                        *interface = requested_interface;
+                        *interface = PQOS_INTER_MSR;
+#else
+                *interface = PQOS_INTER_MSR;
+#endif
+        } else {
+                *interface = requested_interface;
         }
 
-        LOG_INFO("Selected interface: %s\n", interface_to_string(*interface));
+        LOG_INFO("Selected interface: %s\n",
+                 _cap_interface_to_string(*interface));
         return PQOS_RETVAL_OK;
 }
 
@@ -697,16 +621,17 @@ pqos_init(const struct pqos_config *config)
         if (config == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        if (_pqos_api_init() != 0) {
+        if (lock_init() != 0) {
                 fprintf(stderr, "API lock initialization error!\n");
                 return PQOS_RETVAL_ERROR;
         }
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(0);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
+                lock_fini();
                 return ret;
         }
 
@@ -775,11 +700,11 @@ pqos_init(const struct pqos_config *config)
 
         ret = api_init(interface, m_cpu->vendor);
         if (ret != PQOS_RETVAL_OK) {
-                LOG_ERROR("_pqos_api_init() error %d\n", ret);
+                LOG_ERROR("lock_init() error %d\n", ret);
                 goto machine_init_error;
         }
 
-        m_interface = interface;
+        _pqos_set_inter(interface);
 
         ret = pqos_alloc_init(m_cpu, m_cap, config);
         switch (ret) {
@@ -845,10 +770,10 @@ init_error:
         if (ret == PQOS_RETVAL_OK)
                 m_init_done = 1;
 
-        _pqos_api_unlock();
+        lock_release();
 
         if (ret != PQOS_RETVAL_OK)
-                _pqos_api_exit();
+                lock_fini();
 
         return ret;
 }
@@ -860,12 +785,12 @@ pqos_fini(void)
         int retval = PQOS_RETVAL_OK;
         unsigned i = 0;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
-                _pqos_api_exit();
+                lock_release();
+                lock_fini();
                 return ret;
         }
 
@@ -890,16 +815,18 @@ pqos_fini(void)
 
         m_cpu = NULL;
 
-        for (i = 0; i < m_cap->num_cap; i++)
-                free(m_cap->capabilities[i].u.generic_ptr);
-        free((void *)m_cap);
-        m_cap = NULL;
+        if (m_cap != NULL) {
+                for (i = 0; i < m_cap->num_cap; i++)
+                        free(m_cap->capabilities[i].u.generic_ptr);
+                free((void *)m_cap);
+                m_cap = NULL;
+        };
 
         m_init_done = 0;
 
-        _pqos_api_unlock();
+        lock_release();
 
-        if (_pqos_api_exit() != 0)
+        if (lock_fini() != 0)
                 retval = PQOS_RETVAL_ERROR;
 
         return retval;
@@ -923,17 +850,24 @@ pqos_cap_get(const struct pqos_cap **cap, const struct pqos_cpuinfo **cpu)
         if (cap == NULL && cpu == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
-        _pqos_cap_get(cap, cpu);
+        if (cap != NULL) {
+                *cap = _pqos_get_cap();
+                ASSERT(*cap != NULL);
+        }
+        if (cpu != NULL) {
+                *cpu = _pqos_get_cpu();
+                ASSERT(*cpu != NULL);
+        }
 
-        _pqos_api_unlock();
+        lock_release();
         return PQOS_RETVAL_OK;
 }
 
@@ -1096,25 +1030,44 @@ _pqos_cap_mba_change(const enum pqos_mba_config cfg)
         }
 }
 
-void
-_pqos_cap_get(const struct pqos_cap **cap, const struct pqos_cpuinfo **cpu)
+const struct pqos_cap *
+_pqos_get_cap(void)
 {
-        if (cap != NULL) {
-                ASSERT(m_cap != NULL);
-                *cap = m_cap;
-        }
+        ASSERT(m_cap != NULL);
+        return m_cap;
+}
 
-        if (cpu != NULL) {
-                ASSERT(m_cpu != NULL);
-                *cpu = m_cpu;
-        }
+const struct pqos_cpuinfo *
+_pqos_get_cpu(void)
+{
+        ASSERT(m_cpu != NULL);
+        return m_cpu;
 }
 
 int
-_pqos_cap_get_type(const enum pqos_cap_type type,
-                   const struct pqos_capability **cap_item)
+cap_get_l3ca_non_contignous(void)
 {
-        return pqos_cap_get_type(m_cap, type, cap_item);
+        const struct pqos_capability *l3cap =
+            _pqos_cap_get_type(PQOS_CAP_TYPE_L3CA);
+
+        /* If L3 capability is not available, return contiguous CBM support */
+        if (l3cap == NULL)
+                return !PQOS_CPUID_CAT_NON_CONTIGUOUS_CBM_SUPPORT;
+
+        return l3cap->u.l3ca->non_contiguous_cbm;
+}
+
+int
+cap_get_l2ca_non_contignous(void)
+{
+        const struct pqos_capability *l2cap =
+            _pqos_cap_get_type(PQOS_CAP_TYPE_L2CA);
+
+        /* If L2 capability is not available, return contiguous CBM support */
+        if (l2cap == NULL)
+                return !PQOS_CPUID_CAT_NON_CONTIGUOUS_CBM_SUPPORT;
+
+        return l2cap->u.l2ca->non_contiguous_cbm;
 }
 
 /**
@@ -1135,22 +1088,16 @@ pqos_inter_get(enum pqos_interface *interface)
         if (interface == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_api_lock();
+        lock_get();
 
         ret = _pqos_check_init(1);
         if (ret != PQOS_RETVAL_OK) {
-                _pqos_api_unlock();
+                lock_release();
                 return ret;
         }
 
         *interface = _pqos_get_inter();
 
-        _pqos_api_unlock();
+        lock_release();
         return PQOS_RETVAL_OK;
 }
-
-enum pqos_interface
-_pqos_get_inter(void)
-{
-        return m_interface;
-}
