@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2019-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2019-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,15 +31,15 @@
  *
  */
 
-#include <stdio.h>
-#include <sys/stat.h>
-#include <string.h>
-#include <stdlib.h>
-#include <ctype.h>
-
-#include "pqos.h"
 #include "common.h"
+
 #include "log.h"
+#include "pqos.h"
+
+#include <ctype.h>
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
 
 FILE *
 pqos_fopen(const char *name, const char *mode)
@@ -83,6 +83,12 @@ pqos_fopen_error:
         return NULL;
 }
 
+int
+pqos_fclose(FILE *stream)
+{
+        return fclose(stream);
+}
+
 char *
 pqos_strcat(char *dst, const char *src, size_t size)
 {
@@ -125,11 +131,37 @@ pqos_fgets_error:
         return NULL;
 }
 
+/**
+ * @brief Read integer from file
+ *
+ * @param [in] path file path
+ * @param [out] value parsed value
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int
+pqos_fread_uint(const char *path, unsigned *value)
+{
+        int ret = PQOS_RETVAL_OK;
+        FILE *fd;
+
+        fd = pqos_fopen(path, "r");
+        if (fd != NULL) {
+                if (fscanf(fd, "%u", value) != 1)
+                        ret = PQOS_RETVAL_ERROR;
+                pqos_fclose(fd);
+        } else
+                return PQOS_RETVAL_RESOURCE;
+
+        return ret;
+}
+
 int
 pqos_fread_uint64(const char *fname, unsigned base, uint64_t *value)
 {
         FILE *fd;
-        char buf[16] = "\0";
+        char buf[17] = "\0";
         char *s = buf;
         char *endptr = NULL;
         size_t bytes;
@@ -144,10 +176,10 @@ pqos_fread_uint64(const char *fname, unsigned base, uint64_t *value)
 
         bytes = fread(buf, sizeof(buf) - 1, 1, fd);
         if (bytes == 0 && !feof(fd)) {
-                fclose(fd);
+                pqos_fclose(fd);
                 return PQOS_RETVAL_ERROR;
         }
-        fclose(fd);
+        pqos_fclose(fd);
 
         val = strtoull(s, &endptr, base);
 
@@ -211,6 +243,7 @@ pqos_file_contains(const char *fname, const char *str, int *found)
                 return PQOS_RETVAL_OK;
         }
 
+        *found = 0;
         while (fgets(temp, sizeof(temp), fd) != NULL) {
                 if (strstr(temp, str) != NULL) {
                         *found = 1;
