@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2019-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2019-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -61,6 +61,16 @@ extern "C" {
  */
 PQOS_LOCAL FILE *pqos_fopen(const char *name, const char *mode);
 
+/**
+ * @brief Wrapper around fclose()
+ *
+ * @param [in] stream file descriptor
+ *
+ * @return Operational status
+ * @retval 0 on success
+ */
+PQOS_LOCAL int pqos_fclose(FILE *stream);
+
 /**
  * @brief Wrapper around strcat
  *
@@ -77,6 +87,17 @@ PQOS_LOCAL char *pqos_strcat(char *dst, const char *src, size_t size);
  */
 PQOS_LOCAL char *pqos_fgets(char *s, int n, FILE *stream);
 
+/**
+ * @brief Read integer from file
+ *
+ * @param [in] path file path
+ * @param [out] value parsed value
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+PQOS_LOCAL int pqos_fread_uint(const char *path, unsigned *value);
+
 /**
  * @brief Read uint64 from file
  *
