@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -56,6 +56,7 @@
 #endif
 #ifdef __linux__
 #include <sched.h> /* sched affinity */
+#include <sys/syscall.h>
 #endif
 
 /**
@@ -398,9 +399,25 @@ detect_cpu(const int cpu,
         info->l3_id = apicid >> apic->l3_shift;
         info->l2_id = apicid >> apic->l2_shift;
 
+#if (PQOS_VERSION >= 50000 || defined PQOS_SNC)
+#if defined __linux__ && defined __NR_getcpu
+        syscall(__NR_getcpu, NULL, &info->numa, NULL);
+#else
+        if (getcpu(NULL, &info->numa) != 0)
+                return -1;
+#endif
+#endif
+
+#if (PQOS_VERSION >= 50000 || defined PQOS_SNC)
+        LOG_DEBUG("Detected core %u, socket %u, NUMAnode %u, "
+                  "L2 ID %u, L3 ID %u, APICID %u\n",
+                  info->lcore, info->socket, info->numa, info->l2_id,
+                  info->l3_id, apicid);
+#else
         LOG_DEBUG("Detected core %u, socket %u, "
                   "L2 ID %u, L3 ID %u, APICID %u\n",
                   info->lcore, info->socket, info->l2_id, info->l3_id, apicid);
+#endif
 
         return 0;
 }
@@ -497,6 +514,48 @@ detect_vendor(void)
         return cpu_vendor;
 }
 
+/**
+ * @brief Detect cpu model
+ *
+ * @return detected cpu model
+ */
+uint32_t
+cpuinfo_get_cpu_model(void)
+{
+        uint32_t model;
+        struct cpuid_out out;
+
+        lcpuid(1, 0, &out);
+
+        /* Read CPU model */
+        model = (out.eax & 0xf0) >> 4;
+        /* Read CPU extended model */
+        model |= (out.eax & 0xf0000) >> 12;
+
+        return model;
+}
+
+/**
+ * @brief Detect cpu family
+ *
+ * @return detected cpu family
+ */
+uint32_t
+cpuinfo_get_cpu_family(void)
+{
+        uint32_t family;
+        struct cpuid_out out;
+
+        lcpuid(1, 0, &out);
+
+        /* Read CPU family */
+        family = (out.eax & 0xf00) >> 8;
+        /* Read CPU extended family */
+        family |= (out.eax & 0xff00000) >> 16;
+
+        return family;
+}
+
 /**
  * Initialize pointers for the vendors
  *
