@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -47,6 +47,10 @@
 extern "C" {
 #endif
 
+#define CPU_MODEL_SKX 0x55
+#define CPU_MODEL_HSX 0x3f
+#define CPU_FAMILY_06 0x06
+
 /**
  * CPU vendor configuration value
  */
@@ -90,6 +94,20 @@ PQOS_LOCAL int cpuinfo_fini(void);
  */
 PQOS_LOCAL void cpuinfo_get_config(const struct cpuinfo_config **config);
 
+/**
+ * @brief Detect cpu model
+ *
+ * @return detected cpu model
+ */
+uint32_t cpuinfo_get_cpu_model(void);
+
+/**
+ * @brief Detect cpu family
+ *
+ * @return detected cpu family
+ */
+uint32_t cpuinfo_get_cpu_family(void);
+
 #ifdef __cplusplus
 }
 #endif
