@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -33,14 +33,17 @@
 /**
  * @brief HW implementation of PQoS API / capabilities.
  */
-#include <stdlib.h>
-#include <string.h>
+
+#include "hw_cap.h"
 
 #include "cpu_registers.h"
-#include "hw_cap.h"
+#include "cpuinfo.h"
 #include "log.h"
 #include "machine.h"
-#include "types.h"
+#include "uncore_monitoring.h"
+
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * @brief Retrieves cache size and number of ways
@@ -102,7 +105,7 @@ add_monitoring_event(struct pqos_cap_mon *mon,
         }
 
         LOG_DEBUG("Adding monitoring event: resource ID %u, "
-                  "type %d to table index %u\n",
+                  "type %x to table index %u\n",
                   res_id, event_type, mon->num_events);
 
         mon->events[mon->num_events].type = (enum pqos_mon_event)event_type;
@@ -136,6 +139,7 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
         int ret = PQOS_RETVAL_OK;
         unsigned sz = 0, max_rmid = 0, l3_size = 0, num_events = 0;
         struct pqos_cap_mon *mon = NULL;
+        enum pqos_mon_event uncore_events = (enum pqos_mon_event)0;
 
         ASSERT(r_mon != NULL && cpu != NULL);
 
@@ -184,6 +188,21 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
         if ((cpuid_0xf_1.edx & 2) && (cpuid_0xf_1.edx & 4))
                 num_events++; /**< remote memory bandwidth virtual event */
 
+        ret = uncore_mon_discover(&uncore_events);
+        if (ret == PQOS_RETVAL_RESOURCE)
+                LOG_DEBUG("Uncore monitoring not detected\n");
+        else if (ret == PQOS_RETVAL_OK) {
+                if (uncore_events & PQOS_PERF_EVENT_LLC_MISS_PCIE_READ)
+                        num_events++;
+                if (uncore_events & PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE)
+                        num_events++;
+                if (uncore_events & PQOS_PERF_EVENT_LLC_REF_PCIE_READ)
+                        num_events++;
+                if (uncore_events & PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE)
+                        num_events++;
+        } else
+                return ret;
+
         if (!num_events)
                 return PQOS_RETVAL_ERROR;
 
@@ -195,10 +214,10 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
                 num_events++;
 
         /**
-         * This means we can program LLC misses too
+         * This means we can program LLC misses and references too
          */
         if (((cpuid_0xa.eax >> 8) & 0xff) > 1)
-                num_events++;
+                num_events += 2;
 
         /**
          * Allocate memory for detected events and
@@ -242,9 +261,26 @@ hw_cap_mon_discover(struct pqos_cap_mon **r_mon, const struct pqos_cpuinfo *cpu)
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_IPC, 0, 0, 0,
                                      num_events);
 
-        if (((cpuid_0xa.eax >> 8) & 0xff) > 1)
+        if (((cpuid_0xa.eax >> 8) & 0xff) > 1) {
                 add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_MISS, 0, 0, 0,
                                      num_events);
+                add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_REF, 0, 0, 0,
+                                     num_events);
+        }
+
+        if (uncore_events & PQOS_PERF_EVENT_LLC_MISS_PCIE_READ)
+                add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_MISS_PCIE_READ,
+                                     0, 0, 0, num_events);
+        if (uncore_events & PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE)
+                add_monitoring_event(mon, 0,
+                                     PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE, 0, 0,
+                                     0, num_events);
+        if (uncore_events & PQOS_PERF_EVENT_LLC_REF_PCIE_READ)
+                add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_REF_PCIE_READ,
+                                     0, 0, 0, num_events);
+        if (uncore_events & PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE)
+                add_monitoring_event(mon, 0, PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE,
+                                     0, 0, 0, num_events);
 
         (*r_mon) = mon;
         return PQOS_RETVAL_OK;
@@ -437,6 +473,13 @@ hw_cap_l3ca_brandstr(struct pqos_cap_l3ca *cap)
                 LOG_WARN("Cache allocation not supported on model name '%s'!\n",
                          brand_str);
                 return PQOS_RETVAL_RESOURCE;
+        } else {
+                LOG_WARN(
+                    "Detected model specific non-architectural features (L3 "
+                    "CAT).\n      Intel recommends validating that the feature "
+                    "provides the\n      performance necessary for your "
+                    "use-case. Non-architectural\n      features may not "
+                    "behave as expected in all scenarios.\n");
         }
 
         /**
@@ -448,6 +491,34 @@ hw_cap_l3ca_brandstr(struct pqos_cap_l3ca *cap)
         return ret;
 }
 
+/**
+ * @brief Detects presence of L3 CAT based on model & family ID.
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK success
+ * @retval PQOS_RETVAL_RESOURCE technology not supported
+ */
+static int
+hw_cap_l3ca_model(void)
+{
+        const int cpu_model = cpuinfo_get_cpu_model();
+        const int cpu_family = cpuinfo_get_cpu_family();
+
+        const struct {
+                int model;
+                int family;
+        } supported_cpus[] = {
+            {.family = CPU_FAMILY_06, .model = CPU_MODEL_HSX}};
+
+        for (unsigned i = 0; i < DIM(supported_cpus); i++) {
+                if (supported_cpus[i].model == cpu_model &&
+                    supported_cpus[i].family == cpu_family)
+                        return PQOS_RETVAL_OK;
+        };
+
+        return PQOS_RETVAL_RESOURCE;
+}
+
 /**
  * @brief Detects presence of L3 CAT based on CPUID
  *
@@ -485,6 +556,8 @@ hw_cap_l3ca_cpuid(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
         cap->cdp = (res.ecx >> PQOS_CPUID_CAT_CDP_BIT) & 1;
         cap->cdp_on = 0;
         cap->way_contention = (uint64_t)res.ebx;
+        cap->non_contiguous_cbm =
+            (res.ecx >> PQOS_CPUID_CAT_NON_CONTIGUOUS_CBM_SUPPORT_BIT) & 1;
 
         if (cap->cdp) {
                 /**
@@ -528,6 +601,7 @@ hw_cap_l3ca_discover(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
         struct cpuid_out res;
         unsigned l3_size = 0;
         unsigned check_brand_str = 0;
+        unsigned check_cpu_model = 0;
         int ret = PQOS_RETVAL_OK;
 
         ASSERT(cap != NULL);
@@ -546,7 +620,6 @@ hw_cap_l3ca_discover(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
                 /**
                  * Use CPUID method
                  */
-                LOG_INFO("CPUID.0x7.0: L3 CAT supported\n");
                 ret = hw_cap_l3ca_cpuid(cap, cpu);
                 if (ret == PQOS_RETVAL_RESOURCE) {
                         LOG_INFO("CPUID.0x10.0: L3 CAT not detected. "
@@ -557,16 +630,23 @@ hw_cap_l3ca_discover(struct pqos_cap_l3ca *cap, const struct pqos_cpuinfo *cpu)
         } else {
                 LOG_INFO("CPUID.0x7.0: L3 CAT not detected. "
                          "Checking brand string...\n");
-                check_brand_str = 2;
+                check_brand_str = 1;
+                check_cpu_model = 1;
         }
         if (check_brand_str) {
                 /**
                  * Use brand string matching method 1st.
-                 * If it fails then try register probing.
+                 * If it fails then check the model and family ID.
                  */
                 ret = hw_cap_l3ca_brandstr(cap);
-                if (ret != PQOS_RETVAL_OK && check_brand_str > 1)
+                if (ret != PQOS_RETVAL_OK && check_cpu_model) {
+                        LOG_INFO("Checking model and family ID...\n");
+                        ret = hw_cap_l3ca_model();
+                }
+                if (ret != PQOS_RETVAL_OK && getenv("RDT_PROBE_MSR") != NULL) {
+                        LOG_INFO("Probing msr....\n");
                         ret = hw_cap_l3ca_probe(cap, cpu);
+                }
                 if (ret == PQOS_RETVAL_OK)
                         ret =
                             get_cache_info(&cpu->l3, &cap->num_ways, &l3_size);
@@ -678,6 +758,8 @@ hw_cap_l2ca_discover(struct pqos_cap_l2ca *cap, const struct pqos_cpuinfo *cpu)
         cap->cdp = (res.ecx >> PQOS_CPUID_CAT_CDP_BIT) & 1;
         cap->cdp_on = 0;
         cap->way_contention = (uint64_t)res.ebx;
+        cap->non_contiguous_cbm =
+            (res.ecx >> PQOS_CPUID_CAT_NON_CONTIGUOUS_CBM_SUPPORT_BIT) & 1;
 
         if (cap->cdp) {
                 int cdp_on = 0;
