@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -38,16 +38,18 @@
  *
  */
 
-#include <stdlib.h>
-#include <string.h>
+#include "hw_monitoring.h"
 
 #include "cap.h"
 #include "cpu_registers.h"
 #include "log.h"
-#include "hw_monitoring.h"
 #include "machine.h"
 #include "monitoring.h"
 #include "perf_monitoring.h"
+#include "uncore_monitoring.h"
+
+#include <stdlib.h>
+#include <string.h>
 
 /**
  * ---------------------------------------
@@ -75,17 +77,11 @@
  * ---------------------------------------
  */
 static unsigned m_rmid_max = 0; /**< max RMID */
-#ifdef PQOS_RMID_CUSTOM
-/* clang-format off */
-/** Custom RMID configuration */
-static struct pqos_rmid_config rmid_cfg = {PQOS_RMID_TYPE_DEFAULT,
-                                           {0, NULL, NULL} };
-/* clang-format on */
-#endif
 
 /** List of non-virtual perf events */
 static const enum pqos_mon_event perf_event[] = {
-    PQOS_PERF_EVENT_LLC_MISS, (enum pqos_mon_event)PQOS_PERF_EVENT_CYCLES,
+    PQOS_PERF_EVENT_LLC_MISS, PQOS_PERF_EVENT_LLC_REF,
+    (enum pqos_mon_event)PQOS_PERF_EVENT_CYCLES,
     (enum pqos_mon_event)PQOS_PERF_EVENT_INSTRUCTIONS};
 
 /**
@@ -110,9 +106,7 @@ static uint64_t scale_event(const enum pqos_mon_event event,
  */
 
 int
-hw_mon_init(const struct pqos_cpuinfo *cpu,
-            const struct pqos_cap *cap,
-            const struct pqos_config *cfg)
+hw_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 {
         int ret;
         const struct pqos_capability *item = NULL;
@@ -132,38 +126,16 @@ hw_mon_init(const struct pqos_cpuinfo *cpu,
 
 #ifdef __linux__
         ret = perf_mon_init(cpu, cap);
-        /* perf MBM is not supported */
+        if (ret != PQOS_RETVAL_RESOURCE && ret != PQOS_RETVAL_OK)
+                goto hw_mon_init_exit;
+#endif
+
+        ret = uncore_mon_init(cpu, cap);
+        /* uncore is not supported */
         if (ret == PQOS_RETVAL_RESOURCE)
                 ret = PQOS_RETVAL_OK;
         else if (ret != PQOS_RETVAL_OK)
                 goto hw_mon_init_exit;
-#endif
-
-#ifdef PQOS_RMID_CUSTOM
-        rmid_cfg.type = cfg->rmid_cfg.type;
-        if (cfg->rmid_cfg.type == PQOS_RMID_TYPE_MAP) {
-                const unsigned num = cfg->rmid_cfg.map.num;
-                unsigned i;
-
-                if (cfg->rmid_cfg.map.core == NULL ||
-                    cfg->rmid_cfg.map.rmid == NULL) {
-                        ret = PQOS_RETVAL_PARAM;
-                        goto hw_mon_init_exit;
-                }
-
-                rmid_cfg.map.num = num;
-                rmid_cfg.map.core = (unsigned *)malloc(sizeof(unsigned) * num);
-                rmid_cfg.map.rmid =
-                    (pqos_rmid_t *)malloc(sizeof(pqos_rmid_t) * num);
-
-                for (i = 0; i < num; i++) {
-                        rmid_cfg.map.core[i] = cfg->rmid_cfg.map.core[i];
-                        rmid_cfg.map.rmid[i] = cfg->rmid_cfg.map.rmid[i];
-                }
-        }
-#else
-        UNUSED_PARAM(cfg);
-#endif
 
 hw_mon_init_exit:
         if (ret != PQOS_RETVAL_OK)
@@ -177,17 +149,12 @@ hw_mon_fini(void)
 {
         m_rmid_max = 0;
 
+        uncore_mon_fini();
+
 #ifdef __linux__
         perf_mon_fini();
 #endif
 
-#ifdef PQOS_RMID_CUSTOM
-        if (rmid_cfg.map.core != NULL)
-                free(rmid_cfg.map.core);
-        if (rmid_cfg.map.rmid != NULL)
-                free(rmid_cfg.map.rmid);
-#endif
-
         return PQOS_RETVAL_OK;
 }
 
@@ -215,7 +182,7 @@ rmid_get_event_max(const struct pqos_cap *cap,
                    pqos_rmid_t *rmid,
                    const enum pqos_mon_event event)
 {
-        pqos_rmid_t max_rmid = m_rmid_max;
+        pqos_rmid_t max_rmid;
         const struct pqos_capability *item = NULL;
         const struct pqos_cap_mon *mon = NULL;
         unsigned mask_found = 0;
@@ -262,67 +229,94 @@ rmid_get_event_max(const struct pqos_cap *cap,
         return PQOS_RETVAL_OK;
 }
 
-/**
- * @brief Get used RMIDs on ctx->cluster
- *
- * @param [in,out] ctx poll context
- * @param [in] event Monitoring event type
- *
- * @return Operations status
- */
 int
 hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
-                    const enum pqos_mon_event event)
+                    const enum pqos_mon_event event,
+                    pqos_rmid_t min_rmid,
+                    pqos_rmid_t max_rmid,
+                    const struct pqos_mon_options *opt)
 {
-        const struct pqos_cpuinfo *cpu;
-        const struct pqos_cap *cap;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
+        const struct pqos_cap *cap = _pqos_get_cap();
         int ret = PQOS_RETVAL_OK;
-        unsigned max_rmid = 0;
+        pqos_rmid_t rmid;
         unsigned *core_list = NULL;
         unsigned i, core_count;
-        pqos_rmid_t *rmid_list = NULL;
+        uint8_t *rmid_list = NULL;
 
         ASSERT(ctx != NULL);
 
-        _pqos_cap_get(&cap, &cpu);
+#ifndef PQOS_RMID_CUSTOM
+        UNUSED_PARAM(opt);
+#endif
 
         /* Getting max RMID for given event */
-        ret = rmid_get_event_max(cap, &max_rmid, event);
+        ret = rmid_get_event_max(cap, &rmid, event);
         if (ret != PQOS_RETVAL_OK)
                 return ret;
+        if (rmid - 1 < max_rmid)
+                max_rmid = rmid - 1;
+        if (min_rmid < 1)
+                min_rmid = 1;
+
+        /* list of used RMIDs */
+        rmid_list = (uint8_t *)calloc(max_rmid + 2, sizeof(*rmid_list));
+        if (rmid_list == NULL)
+                return PQOS_RETVAL_RESOURCE;
 
         /**
          * Check for free RMID in the cluster by reading current associations.
          */
         core_list = pqos_cpu_get_cores_l3id(cpu, ctx->cluster, &core_count);
-        if (core_list == NULL)
-                return PQOS_RETVAL_ERROR;
-        ASSERT(core_count > 0);
-        rmid_list = (pqos_rmid_t *)malloc(sizeof(rmid_list[0]) * core_count);
-        if (rmid_list == NULL) {
-                ret = PQOS_RETVAL_RESOURCE;
+        if (core_list == NULL) {
+                ret = PQOS_RETVAL_ERROR;
                 goto rmid_alloc_error;
         }
-
+        ASSERT(core_count > 0);
+        /* Mark RMIDs used for core monitoring */
         for (i = 0; i < core_count; i++) {
-                ret = hw_mon_assoc_read(core_list[i], &rmid_list[i]);
+                pqos_rmid_t rmid;
+
+                ret = hw_mon_assoc_read(core_list[i], &rmid);
                 if (ret != PQOS_RETVAL_OK)
                         goto rmid_alloc_error;
+                if (rmid <= max_rmid)
+                        rmid_list[rmid] = 1;
         }
 
-        ret = PQOS_RETVAL_ERROR;
-        for (i = 1; i < max_rmid; i++) {
-                unsigned j = 0;
-
-                for (j = 0; j < core_count; j++)
-                        if (i == rmid_list[j])
-                                break;
-                if (j >= core_count) {
-                        ret = PQOS_RETVAL_OK;
-                        ctx->rmid = i;
-                        break;
+#ifdef PQOS_RMID_CUSTOM
+        if (opt->rmid.type == PQOS_RMID_TYPE_MAP) {
+                if (opt->rmid.rmid < min_rmid || opt->rmid.rmid > max_rmid) {
+                        LOG_ERROR("Custom RMID %u not in range %u-%u\n",
+                                  opt->rmid.rmid, min_rmid, max_rmid);
+                        ret = PQOS_RETVAL_PARAM;
+                        goto rmid_alloc_error;
                 }
+
+                if (opt->rmid.rmid > max_rmid ||
+                    rmid_list[opt->rmid.rmid] != 0) {
+                        LOG_ERROR("Custom RMID %u in use\n", opt->rmid.rmid);
+                        ret = PQOS_RETVAL_ERROR;
+                        goto rmid_alloc_error;
+                }
+
+                ctx->rmid = opt->rmid.rmid;
+
+        } else if (opt->rmid.type == PQOS_RMID_TYPE_DEFAULT) {
+#endif
+                ret = PQOS_RETVAL_ERROR;
+                for (i = min_rmid; i <= max_rmid; i++)
+                        if (rmid_list[i] == 0) {
+                                ret = PQOS_RETVAL_OK;
+                                ctx->rmid = i;
+                                break;
+                        }
+#ifdef PQOS_RMID_CUSTOM
+        } else {
+                LOG_ERROR("RMID Custom: Unsupported rmid type: %u\n",
+                          opt->rmid.type);
         }
+#endif
 
 rmid_alloc_error:
         if (rmid_list != NULL)
@@ -332,49 +326,6 @@ rmid_alloc_error:
         return ret;
 }
 
-#ifdef PQOS_RMID_CUSTOM
-/**
- * @brief Gets RMID value based on information stored in rmid_cfg
- *
- * @param [in,out] ctx poll context
- * @param [in] event Monitoring event type
- * @param [in] rmid_cfg rmid configuration parameters
- *
- * @return Operation status
- * @retval PQOS_RETVAL_OK on success
- */
-static int
-hw_mon_assoc_unused_custom(struct pqos_mon_poll_ctx *ctx,
-                           const enum pqos_mon_event event,
-                           const struct pqos_rmid_config *rmid_cfg)
-{
-        if (ctx == NULL)
-                return PQOS_RETVAL_PARAM;
-
-        if (rmid_cfg == NULL || rmid_cfg->type == PQOS_RMID_TYPE_DEFAULT) {
-                return hw_mon_assoc_unused(ctx, event);
-        } else if (rmid_cfg->type == PQOS_RMID_TYPE_MAP) {
-                unsigned i;
-
-                for (i = 0; i < rmid_cfg->map.num; i++) {
-                        if (ctx->lcore == rmid_cfg->map.core[i]) {
-                                ctx->rmid = rmid_cfg->map.rmid[i];
-                                return PQOS_RETVAL_OK;
-                        }
-                }
-
-                LOG_ERROR("RMID Custom: No mapping for core %u\n", ctx->lcore);
-        } else {
-                LOG_ERROR("RMID Custom: Unsupported rmid type: %u\n",
-                          rmid_cfg->type);
-
-                return PQOS_RETVAL_PARAM;
-        }
-
-        return PQOS_RETVAL_ERROR;
-}
-#endif
-
 /*
  * =======================================
  * =======================================
@@ -399,12 +350,10 @@ hw_mon_assoc_unused_custom(struct pqos_mon_poll_ctx *ctx,
 static uint64_t
 scale_event(const enum pqos_mon_event event, const uint64_t val)
 {
-        const struct pqos_cap *cap;
+        const struct pqos_cap *cap = _pqos_get_cap();
         const struct pqos_monitor *pmon;
         int ret;
 
-        _pqos_cap_get(&cap, NULL);
-
         ASSERT(cap != NULL);
 
         ret = pqos_cap_get_event(cap, event, &pmon);
@@ -460,13 +409,11 @@ int
 hw_mon_assoc_get(const unsigned lcore, pqos_rmid_t *rmid)
 {
         int ret = PQOS_RETVAL_OK;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         if (rmid == NULL)
                 return PQOS_RETVAL_PARAM;
 
-        _pqos_cap_get(NULL, &cpu);
-
         ret = pqos_cpu_check_core(cpu, lcore);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_PARAM;
@@ -481,9 +428,15 @@ hw_mon_reset(void)
 {
         int ret = PQOS_RETVAL_OK;
         unsigned i;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_capability *cap_mon;
 
-        _pqos_cap_get(NULL, &cpu);
+        ret = pqos_cap_get_type(cap, PQOS_CAP_TYPE_MON, &cap_mon);
+        if (ret != PQOS_RETVAL_OK) {
+                LOG_ERROR("Monitoring not present!\n");
+                return ret;
+        }
 
         for (i = 0; i < cpu->num_cores; i++) {
                 int retval = hw_mon_assoc_write(cpu->cores[i].lcore, RMID0);
@@ -571,13 +524,11 @@ get_delta(const enum pqos_mon_event event,
           const uint64_t old_value,
           const uint64_t new_value)
 {
-        const struct pqos_cap *cap;
+        const struct pqos_cap *cap = _pqos_get_cap();
         const struct pqos_monitor *pmon;
         int ret;
         uint64_t max_value = 1LLU << 24;
 
-        _pqos_cap_get(&cap, NULL);
-
         ret = pqos_cap_get_event(cap, event, &pmon);
         if (ret == PQOS_RETVAL_OK)
                 max_value = 1LLU << pmon->counter_length;
@@ -608,14 +559,18 @@ ia32_perf_counter_start(const struct pqos_mon_data *group,
 
         ASSERT(cores != NULL && num_cores > 0);
 
-        if (!(event & (PQOS_PERF_EVENT_LLC_MISS | PQOS_PERF_EVENT_IPC)))
+        if (!(event & (PQOS_PERF_EVENT_LLC_MISS | PQOS_PERF_EVENT_LLC_REF |
+                       PQOS_PERF_EVENT_IPC)))
                 return PQOS_RETVAL_OK;
 
         if (event & PQOS_PERF_EVENT_IPC)
-                global_ctrl_mask |= (0x3ULL << 32); /**< fixed counters 0&1 */
+                global_ctrl_mask |= (0x3ULL << 32); /* fixed counters 0&1 */
 
         if (event & PQOS_PERF_EVENT_LLC_MISS)
-                global_ctrl_mask |= 0x1ULL; /**< programmable counter 0 */
+                global_ctrl_mask |= 0x1ULL; /* programmable counter 0 */
+
+        if (event & PQOS_PERF_EVENT_LLC_REF)
+                global_ctrl_mask |= (0x1ULL << 1); /* programmable counter 1 */
 
         /**
          * Fixed counters are used for IPC calculations.
@@ -678,6 +633,21 @@ ia32_perf_counter_start(const struct pqos_mon_data *group,
                                 break;
                 }
 
+                if (event & PQOS_PERF_EVENT_LLC_REF) {
+                        const uint64_t evtsel0_ref =
+                            IA32_EVENT_LLC_REF_MASK |
+                            (IA32_EVENT_LLC_REF_UMASK << 8) | (1ULL << 16) |
+                            (1ULL << 17) | (1ULL << 22);
+
+                        ret = msr_write(cores[i], IA32_MSR_PMC1, 0);
+                        if (ret != MACHINE_RETVAL_OK)
+                                break;
+                        ret = msr_write(cores[i], IA32_MSR_PERFEVTSEL1,
+                                        evtsel0_ref);
+                        if (ret != MACHINE_RETVAL_OK)
+                                break;
+                }
+
                 ret = msr_write(cores[i], IA32_MSR_PERF_GLOBAL_CTRL,
                                 global_ctrl_mask);
                 if (ret != MACHINE_RETVAL_OK)
@@ -710,7 +680,8 @@ ia32_perf_counter_stop(const unsigned num_cores,
 
         ASSERT(cores != NULL && num_cores > 0);
 
-        if (!(event & (PQOS_PERF_EVENT_LLC_MISS | PQOS_PERF_EVENT_IPC)))
+        if (!(event & (PQOS_PERF_EVENT_LLC_MISS | PQOS_PERF_EVENT_LLC_REF |
+                       PQOS_PERF_EVENT_IPC)))
                 return retval;
 
         for (i = 0; i < num_cores; i++) {
@@ -809,10 +780,12 @@ hw_mon_stop_perf(struct pqos_mon_data *group)
 }
 
 int
-hw_mon_start_counter(struct pqos_mon_data *group, enum pqos_mon_event event)
+hw_mon_start_counter(struct pqos_mon_data *group,
+                     enum pqos_mon_event event,
+                     const struct pqos_mon_options *opt)
 {
         const unsigned num_cores = group->num_cores;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         unsigned core2cluster[num_cores];
         struct pqos_mon_poll_ctx ctxs[num_cores];
         unsigned num_ctxs = 0;
@@ -822,8 +795,6 @@ hw_mon_start_counter(struct pqos_mon_data *group, enum pqos_mon_event event)
             event & (PQOS_MON_EVENT_L3_OCCUP | PQOS_MON_EVENT_LMEM_BW |
                      PQOS_MON_EVENT_TMEM_BW | PQOS_MON_EVENT_RMEM_BW));
 
-        _pqos_cap_get(NULL, &cpu);
-
         memset(ctxs, 0, sizeof(ctxs));
 
         /*
@@ -854,12 +825,9 @@ hw_mon_start_counter(struct pqos_mon_data *group, enum pqos_mon_event event)
                          */
                         ctxs[num_ctxs].lcore = lcore;
                         ctxs[num_ctxs].cluster = cluster;
-#ifdef PQOS_RMID_CUSTOM
-                        ret = hw_mon_assoc_unused_custom(&ctxs[num_ctxs],
-                                                         ctx_event, &rmid_cfg);
-#else
-                        ret = hw_mon_assoc_unused(&ctxs[num_ctxs], ctx_event);
-#endif
+
+                        ret = hw_mon_assoc_unused(&ctxs[num_ctxs], ctx_event, 1,
+                                                  UINT32_MAX, opt);
                         if (ret != PQOS_RETVAL_OK)
                                 return ret;
 
@@ -914,18 +882,50 @@ hw_mon_start_counter_exit:
         return ret;
 }
 
+/**
+ * @brief Validate if event is listed in capabilities
+ *
+ * @param [in[ cap capabilities structure
+ * @param [in] event monitoring event id
+ *
+ * @return Operational status
+ * @retval PQOS_RETVAL_OK success
+ */
+static int
+validate_event(const struct pqos_cap *cap, const enum pqos_mon_event event)
+{
+        unsigned i;
+
+        for (i = 0; i < (sizeof(event) * 8); i++) {
+                const enum pqos_mon_event evt_mask =
+                    (enum pqos_mon_event)(1U << i);
+                const struct pqos_monitor *ptr = NULL;
+                int retval;
+
+                if (!(evt_mask & event))
+                        continue;
+
+                retval = pqos_cap_get_event(cap, evt_mask, &ptr);
+                if (retval != PQOS_RETVAL_OK || ptr == NULL)
+                        return PQOS_RETVAL_PARAM;
+        }
+
+        return PQOS_RETVAL_OK;
+}
+
 int
-hw_mon_start(const unsigned num_cores,
-             const unsigned *cores,
-             const enum pqos_mon_event event,
-             void *context,
-             struct pqos_mon_data *group)
+hw_mon_start_cores(const unsigned num_cores,
+                   const unsigned *cores,
+                   const enum pqos_mon_event event,
+                   void *context,
+                   struct pqos_mon_data *group,
+                   const struct pqos_mon_options *opt)
 {
         unsigned i;
         int ret = PQOS_RETVAL_OK;
         int retval = PQOS_RETVAL_OK;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
         enum pqos_mon_event req_events;
         enum pqos_mon_event started_evts = (enum pqos_mon_event)0;
 
@@ -934,8 +934,6 @@ hw_mon_start(const unsigned num_cores,
         ASSERT(num_cores > 0);
         ASSERT(event > 0);
 
-        _pqos_cap_get(&cap, &cpu);
-
         req_events = event;
 
         if (req_events & PQOS_MON_EVENT_RMEM_BW)
@@ -945,21 +943,9 @@ hw_mon_start(const unsigned num_cores,
                 req_events |= (enum pqos_mon_event)(
                     PQOS_PERF_EVENT_CYCLES | PQOS_PERF_EVENT_INSTRUCTIONS);
 
-        /**
-         * Validate if event is listed in capabilities
-         */
-        for (i = 0; i < (sizeof(event) * 8); i++) {
-                const enum pqos_mon_event evt_mask =
-                    (enum pqos_mon_event)(1U << i);
-                const struct pqos_monitor *ptr = NULL;
-
-                if (!(evt_mask & event))
-                        continue;
-
-                retval = pqos_cap_get_event(cap, evt_mask, &ptr);
-                if (retval != PQOS_RETVAL_OK || ptr == NULL)
-                        return PQOS_RETVAL_PARAM;
-        }
+        ret = validate_event(cap, event);
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
 
         /**
          * Check if all requested cores are valid
@@ -981,6 +967,8 @@ hw_mon_start(const unsigned num_cores,
                         return PQOS_RETVAL_PARAM;
 
                 if (rmid != RMID0) {
+                        LOG_ERROR("Monitoring on core %u is already started\n",
+                                  lcore);
                         /* If not RMID0 then it is already monitored */
                         LOG_INFO("Core %u is already monitored with "
                                  "RMID%u.\n",
@@ -1007,7 +995,7 @@ hw_mon_start(const unsigned num_cores,
                 goto pqos_mon_start_error;
 
         /* start MBM/CMT events */
-        retval = hw_mon_start_counter(group, req_events);
+        retval = hw_mon_start_counter(group, req_events, opt);
         if (retval != PQOS_RETVAL_OK)
                 goto pqos_mon_start_error;
 
@@ -1050,22 +1038,90 @@ pqos_mon_start_error:
         return retval;
 }
 
+int
+hw_mon_start_uncore(const unsigned num_sockets,
+                    const unsigned *sockets,
+                    const enum pqos_mon_event event,
+                    void *context,
+                    struct pqos_mon_data *group)
+{
+        int ret;
+        unsigned i;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
+        enum pqos_mon_event started_evts = (enum pqos_mon_event)0;
+
+        ASSERT(group != NULL);
+        ASSERT(sockets != NULL);
+        ASSERT(num_sockets > 0);
+        ASSERT(event > 0);
+
+        ret = validate_event(cap, event);
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
+
+        /* Check if all requested sockets are valid */
+        for (i = 0; i < num_sockets; i++) {
+                unsigned lcore;
+
+                ret = pqos_cpu_get_one_core(cpu, sockets[i], &lcore);
+                if (ret != PQOS_RETVAL_OK)
+                        return PQOS_RETVAL_PARAM;
+        }
+
+        /**
+         * Fill in the monitoring group structure
+         */
+        group->event = event;
+        group->context = context;
+        group->intl->uncore.num_sockets = num_sockets;
+        group->intl->uncore.sockets = (unsigned *)malloc(
+            sizeof(group->intl->uncore.sockets[0]) * num_sockets);
+        if (group->intl->uncore.sockets == NULL)
+                return PQOS_RETVAL_RESOURCE;
+        for (i = 0; i < num_sockets; i++)
+                group->intl->uncore.sockets[i] = sockets[i];
+
+        ret = uncore_mon_start(group, event);
+        if (ret != PQOS_RETVAL_OK)
+                goto hw_mon_start_uncore__exit;
+
+        started_evts |= group->intl->hw.event;
+
+        /*  Check if all selected events were started */
+        if ((group->event & started_evts) != group->event) {
+                LOG_ERROR("Failed to start all selected "
+                          "HW monitoring events\n");
+                ret = PQOS_RETVAL_ERROR;
+        }
+
+hw_mon_start_uncore__exit:
+        if (ret != PQOS_RETVAL_OK) {
+                uncore_mon_stop(group);
+
+                if (group->intl->uncore.sockets != NULL)
+                        free(group->intl->uncore.sockets);
+        }
+
+        return ret;
+}
+
 int
 hw_mon_stop(struct pqos_mon_data *group)
 {
         int ret = PQOS_RETVAL_OK;
         int retval = PQOS_RETVAL_OK;
         unsigned i = 0;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
 
         ASSERT(group != NULL);
 
-        if (group->num_cores == 0 || group->cores == NULL ||
-            group->intl->hw.num_ctx == 0 || group->intl->hw.ctx == NULL) {
+        if (group->num_cores == 0 && group->intl->uncore.num_sockets == 0) {
+                return PQOS_RETVAL_PARAM;
+        } else if (group->num_cores > 0 &&
+                   (group->cores == NULL || group->intl->hw.num_ctx == 0 ||
+                    group->intl->hw.ctx == NULL))
                 return PQOS_RETVAL_PARAM;
-        }
-
-        _pqos_cap_get(NULL, &cpu);
 
         for (i = 0; i < group->intl->hw.num_ctx; i++) {
                 /**
@@ -1100,12 +1156,16 @@ hw_mon_stop(struct pqos_mon_data *group)
         if (ret != PQOS_RETVAL_OK)
                 retval = ret;
 
+        /* stop uncore counters */
+        ret = uncore_mon_stop(group);
+        if (ret != PQOS_RETVAL_OK)
+                retval = ret;
+
         /**
          * Free poll contexts, core list and clear the group structure
          */
         free(group->cores);
         free(group->intl->hw.ctx);
-        memset(group, 0, sizeof(*group));
 
         return retval;
 }
@@ -1117,7 +1177,7 @@ hw_mon_read_counter(struct pqos_mon_data *group,
         struct pqos_event_values *pv = &group->values;
         uint64_t value = 0;
         uint64_t max_value = 1LLU << 24;
-        const struct pqos_cap *cap;
+        const struct pqos_cap *cap = _pqos_get_cap();
         const struct pqos_monitor *pmon;
         unsigned i;
         int ret;
@@ -1126,8 +1186,6 @@ hw_mon_read_counter(struct pqos_mon_data *group,
                event == PQOS_MON_EVENT_LMEM_BW ||
                event == PQOS_MON_EVENT_TMEM_BW);
 
-        _pqos_cap_get(&cap, NULL);
-
         ret = pqos_cap_get_event(cap, event, &pmon);
         if (ret == PQOS_RETVAL_OK)
                 max_value = 1LLU << pmon->counter_length;
@@ -1206,7 +1264,7 @@ hw_mon_read_counter(struct pqos_mon_data *group,
 static int
 hw_mon_read_perf(struct pqos_mon_data *group, const enum pqos_mon_event event)
 {
-        struct pqos_event_values *pv = &group->values;
+        struct pqos_event_values *values = &group->values;
         uint64_t val = 0;
         unsigned n;
         uint64_t reg;
@@ -1216,18 +1274,28 @@ hw_mon_read_perf(struct pqos_mon_data *group, const enum pqos_mon_event event)
         switch (event) {
         case (enum pqos_mon_event)PQOS_PERF_EVENT_INSTRUCTIONS:
                 reg = IA32_MSR_INST_RETIRED_ANY;
-                value = &pv->ipc_retired;
-                delta = &pv->ipc_retired_delta;
+                value = &values->ipc_retired;
+                delta = &values->ipc_retired_delta;
                 break;
         case (enum pqos_mon_event)PQOS_PERF_EVENT_CYCLES:
                 reg = IA32_MSR_CPU_UNHALTED_THREAD;
-                value = &pv->ipc_unhalted;
-                delta = &pv->ipc_unhalted_delta;
+                value = &values->ipc_unhalted;
+                delta = &values->ipc_unhalted_delta;
                 break;
         case PQOS_PERF_EVENT_LLC_MISS:
                 reg = IA32_MSR_PMC0;
-                value = &pv->llc_misses;
-                delta = &pv->llc_misses_delta;
+                value = &values->llc_misses;
+                delta = &values->llc_misses_delta;
+                break;
+        case PQOS_PERF_EVENT_LLC_REF:
+                reg = IA32_MSR_PMC1;
+#if PQOS_VERSION < 50000
+                value = &group->intl->values.llc_references;
+                delta = &group->intl->values.llc_references_delta;
+#else
+                value = &values->llc_references;
+                delta = &values->llc_references_delta;
+#endif
                 break;
         default:
                 return PQOS_RETVAL_PARAM;
@@ -1263,20 +1331,29 @@ hw_mon_poll(struct pqos_mon_data *group, const enum pqos_mon_event event)
         case PQOS_MON_EVENT_TMEM_BW:
                 ret = hw_mon_read_counter(group, event);
                 if (ret != PQOS_RETVAL_OK)
-                        goto pqos_core_poll__exit;
+                        goto hw_mon_poll__exit;
                 break;
         case (enum pqos_mon_event)PQOS_PERF_EVENT_CYCLES:
         case (enum pqos_mon_event)PQOS_PERF_EVENT_INSTRUCTIONS:
         case PQOS_PERF_EVENT_LLC_MISS:
+        case PQOS_PERF_EVENT_LLC_REF:
                 ret = hw_mon_read_perf(group, event);
                 if (ret != PQOS_RETVAL_OK)
-                        goto pqos_core_poll__exit;
+                        goto hw_mon_poll__exit;
+                break;
+        case PQOS_PERF_EVENT_LLC_MISS_PCIE_READ:
+        case PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE:
+        case PQOS_PERF_EVENT_LLC_REF_PCIE_READ:
+        case PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE:
+                ret = uncore_mon_poll(group, event);
+                if (ret != PQOS_RETVAL_OK)
+                        goto hw_mon_poll__exit;
                 break;
         default:
                 ret = PQOS_RETVAL_PARAM;
         }
 
-pqos_core_poll__exit:
+hw_mon_poll__exit:
         return ret;
 }
 /*
