@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -41,9 +41,10 @@
 extern "C" {
 #endif
 
-#include "types.h"
-#include "pqos.h"
 #include "monitoring.h"
+#include "pqos.h"
+#include "pqos_internal.h"
+#include "types.h"
 
 /**
  * @brief Initializes hardware monitoring sub-module of the library (CMT)
@@ -56,8 +57,7 @@ extern "C" {
  * @retval PQOS_RETVAL_OK success
  */
 PQOS_LOCAL int hw_mon_init(const struct pqos_cpuinfo *cpu,
-                           const struct pqos_cap *cap,
-                           const struct pqos_config *cfg);
+                           const struct pqos_cap *cap);
 
 /**
  * @brief Shuts down hardware monitoring sub-module of the library
@@ -105,11 +105,16 @@ PQOS_LOCAL int hw_mon_assoc_read(const unsigned lcore, pqos_rmid_t *rmid);
  *
  * @param [in,out] ctx poll context
  * @param [in] event Monitoring event type
+ * @param [in] min_rmid min RMID
+ * @param [in] max_rmid max RMID
  *
  * @return Operations status
  */
 PQOS_LOCAL int hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
-                                   const enum pqos_mon_event event);
+                                   const enum pqos_mon_event event,
+                                   pqos_rmid_t min_rmid,
+                                   pqos_rmid_t max_rmid,
+                                   const struct pqos_mon_options *opt);
 
 /**
  * @brief Hardware interface to read RMID association of the \a lcore
@@ -149,12 +154,14 @@ PQOS_LOCAL int hw_mon_stop_perf(struct pqos_mon_data *group);
  *
  * @param group monitoring structure
  * @param event PQoS event type
+ * @param [in] opt extended options
  *
  * @return Operation status
  * @retval PQOS_RETVAL_OK on success
  */
 PQOS_LOCAL int hw_mon_start_counter(struct pqos_mon_data *group,
-                                    enum pqos_mon_event event);
+                                    enum pqos_mon_event event,
+                                    const struct pqos_mon_options *opt);
 
 /**
  * @brief Hardware interface to start resource monitoring on selected
@@ -172,15 +179,17 @@ PQOS_LOCAL int hw_mon_start_counter(struct pqos_mon_data *group,
  * @param [in] context a pointer for application's convenience
  *            (unused by the library)
  * @param [in,out] group a pointer to monitoring structure
+ * @param [in] opt extended options
  *
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
  */
-PQOS_LOCAL int hw_mon_start(const unsigned num_cores,
-                            const unsigned *cores,
-                            const enum pqos_mon_event event,
-                            void *context,
-                            struct pqos_mon_data *group);
+PQOS_LOCAL int hw_mon_start_cores(const unsigned num_cores,
+                                  const unsigned *cores,
+                                  const enum pqos_mon_event event,
+                                  void *context,
+                                  struct pqos_mon_data *group,
+                                  const struct pqos_mon_options *opt);
 
 /**
  * @brief Hardware interface to stop resource monitoring data for selected
@@ -211,6 +220,25 @@ PQOS_LOCAL int hw_mon_read(const unsigned lcore,
                            const unsigned event,
                            uint64_t *value);
 
+/**
+ * @brief Hardware interface to start uncore monitoring of selected \a sockets
+ *
+ * @param [in] num_sockets number of sockets in \a sockets array
+ * @param [in] sockets array of socket IDs
+ * @param [in] event monitoring event id
+ * @param [in] context a pointer for application's convenience
+ *             (unused by the library)
+ * @param [out] group a pointer to monitoring structure
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+PQOS_LOCAL int hw_mon_start_uncore(const unsigned num_sockets,
+                                   const unsigned *sockets,
+                                   const enum pqos_mon_event event,
+                                   void *context,
+                                   struct pqos_mon_data *group);
+
 /**
  * @brief Read HW counter
  *
