@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,20 +30,19 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "pqos.h"
-#include "cap.h"
 #include "monitoring.h"
-#include "os_monitoring.h"
+
+#include "cap.h"
 #include "hw_monitoring.h"
+#include "log.h"
+#include "os_monitoring.h"
 #include "perf_monitoring.h"
+#include "types.h"
 #ifdef __linux__
 #include "resctrl.h"
 #include "resctrl_monitoring.h"
 #endif
 
-#include "types.h"
-#include "log.h"
-
 /**
  * ---------------------------------------
  * Local macros
@@ -87,6 +86,8 @@ pqos_mon_init(const struct pqos_cpuinfo *cpu,
         int ret;
         enum pqos_interface interface = _pqos_get_inter();
 
+        UNUSED_PARAM(cfg);
+
         ASSERT(cfg != NULL);
         /**
          * If monitoring capability has been discovered
@@ -109,7 +110,7 @@ pqos_mon_init(const struct pqos_cpuinfo *cpu,
                 return ret;
 #endif
         if (interface == PQOS_INTER_MSR)
-                ret = hw_mon_init(cpu, cap, cfg);
+                ret = hw_mon_init(cpu, cap);
 
 pqos_mon_init_exit:
         return ret;
@@ -146,8 +147,13 @@ pqos_mon_poll_events(struct pqos_mon_data *group)
             PQOS_MON_EVENT_LMEM_BW,
             PQOS_MON_EVENT_TMEM_BW,
             PQOS_PERF_EVENT_LLC_MISS,
+            PQOS_PERF_EVENT_LLC_REF,
             (enum pqos_mon_event)PQOS_PERF_EVENT_CYCLES,
-            (enum pqos_mon_event)PQOS_PERF_EVENT_INSTRUCTIONS};
+            (enum pqos_mon_event)PQOS_PERF_EVENT_INSTRUCTIONS,
+            PQOS_PERF_EVENT_LLC_MISS_PCIE_READ,
+            PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE,
+            PQOS_PERF_EVENT_LLC_REF_PCIE_READ,
+            PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE};
 
 #ifdef __linux__
         if (group->intl->resctrl.event != 0) {
@@ -194,7 +200,7 @@ pqos_mon_poll_events(struct pqos_mon_data *group)
          * Calculate values of virtual events
          */
         if (group->event & PQOS_MON_EVENT_RMEM_BW) {
-                const struct pqos_cap *cap;
+                const struct pqos_cap *cap = _pqos_get_cap();
                 const struct pqos_monitor *pmon;
                 uint64_t max_value = 0;
 
@@ -205,8 +211,6 @@ pqos_mon_poll_events(struct pqos_mon_data *group)
                             group->values.mbm_total_delta -
                             group->values.mbm_local_delta;
 
-                _pqos_cap_get(&cap, NULL);
-
                 ret = pqos_cap_get_event(cap, PQOS_MON_EVENT_RMEM_BW, &pmon);
                 if (ret == PQOS_RETVAL_OK)
                         max_value = 1LLU << pmon->counter_length;
