@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -854,9 +854,7 @@ os_alloc_reset_full_exit:
 }
 
 int
-os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-               const enum pqos_cdp_config l2_cdp_cfg,
-               const enum pqos_mba_config mba_cfg)
+os_alloc_reset(const struct pqos_alloc_config *cfg)
 {
         const struct pqos_capability *alloc_cap = NULL;
         const struct pqos_cap_l3ca *l3_cap = NULL;
@@ -870,6 +868,15 @@ os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
         int mba_changed = 0;
         int ret;
         const struct pqos_cap *cap = _pqos_get_cap();
+        enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
+        enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
+        enum pqos_mba_config mba_cfg = PQOS_MBA_ANY;
+
+        if (cfg != NULL) {
+                l3_cdp_cfg = cfg->l3_cdp;
+                l2_cdp_cfg = cfg->l2_cdp;
+                mba_cfg = cfg->mba;
+        }
 
         ASSERT(l3_cdp_cfg == PQOS_REQUIRE_CDP_ON ||
                l3_cdp_cfg == PQOS_REQUIRE_CDP_OFF ||
@@ -1063,6 +1070,20 @@ os_l3ca_set(const unsigned l3cat_id,
         ASSERT(ca != NULL);
         ASSERT(num_cos != 0);
 
+        /* Check L3 CBM is non-contiguous */
+        if (!cap_get_l3ca_non_contignous()) {
+                unsigned idx;
+                /* Check all COS CBM are contiguous */
+                for (idx = 0; idx < num_cos; idx++) {
+                        if (!IS_CONTIGNOUS(ca[idx])) {
+                                LOG_ERROR("L3 COS%u bit mask is not "
+                                          "contiguous!\n",
+                                          ca[idx].class_id);
+                                return PQOS_RETVAL_PARAM;
+                        }
+                }
+        }
+
         ret = pqos_l3ca_get_cos_num(cap, &l3ca_num);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_RESOURCE; /* L3 CAT not supported */
@@ -1278,6 +1299,20 @@ os_l2ca_set(const unsigned l2id,
         ASSERT(ca != NULL);
         ASSERT(num_cos != 0);
 
+        /* Check L2 CBM is non-contiguous */
+        if (!cap_get_l2ca_non_contignous()) {
+                unsigned idx;
+                /* Check all COS CBM are contiguous */
+                for (idx = 0; idx < num_cos; idx++) {
+                        if (!IS_CONTIGNOUS(ca[idx])) {
+                                LOG_ERROR("L2 COS%u bit mask is not "
+                                          "contiguous!\n",
+                                          ca[idx].class_id);
+                                return PQOS_RETVAL_PARAM;
+                        }
+                }
+        }
+
         ret = pqos_l2ca_get_cos_num(cap, &l2ca_num);
         if (ret != PQOS_RETVAL_OK)
                 return PQOS_RETVAL_RESOURCE; /* L2 CAT not supported */
