@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,8 +42,8 @@
 extern "C" {
 #endif
 
-#include "types.h"
 #include "pqos.h"
+#include "types.h"
 
 /**
  * @brief Initializes resctrl file system used for OS allocation interface
@@ -191,16 +191,12 @@ PQOS_LOCAL int os_alloc_reset_tasks(void);
  * As part of allocation reset MBA reconfiguration can be performed.
  * This can be requested via \a mba_cfg.
  *
- * @param [in] l3_cdp_cfg requested L3 CAT CDP config
- * @param [in] l2_cdp_cfg requested L2 CAT CDP config
- * @param [in] mba_cfg requested MBA config
+ * @param [in] cfg requested configuration
  *
  * @return Operation status
  * @retval PQOS_RETVAL_OK on success
  */
-PQOS_LOCAL int os_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-                              const enum pqos_cdp_config l2_cdp_cfg,
-                              const enum pqos_mba_config mba_cfg);
+PQOS_LOCAL int os_alloc_reset(const struct pqos_alloc_config *cfg);
 
 /**
  * @brief OS interface to set classes of service
@@ -305,7 +301,7 @@ PQOS_LOCAL int os_mba_set(const unsigned mba_id,
 
 /**
  * @brief OS interface to set classes of service defined by \a mba_id
- * @NOTE: This function is specific to AMD
+ * @note: This function is specific to AMD
  *
  * @param [in]  mba_id MBA resource id
  * @param [in]  num_cos number of classes of service at \a ca
@@ -339,7 +335,7 @@ PQOS_LOCAL int os_mba_get(const unsigned mba_id,
 
 /**
  * @brief OS interface to read MBA from \a mba_id
- * @NOTE: This function is specific to AMD
+ * @note: This function is specific to AMD
  *
  * @param [in]  mba_id MBA resource id
  * @param [in]  max_num_cos maximum number of classes of service
