@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,27 +31,25 @@
  *
  */
 
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h> /**< pid_t */
-#include <dirent.h> /**< scandir() */
+#include "os_monitoring.h"
 
-#include "pqos.h"
 #include "cap.h"
 #include "log.h"
-#include "types.h"
 #include "monitoring.h"
-#include "os_monitoring.h"
 #include "perf_monitoring.h"
 #include "resctrl.h"
 #include "resctrl_monitoring.h"
 
+#include <dirent.h> /**< scandir() */
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h> /**< pid_t */
+
 /**
  * ---------------------------------------
  * Local data structures
  * ---------------------------------------
  */
-static const struct pqos_cpuinfo *m_cpu = NULL;
 
 /** List of non virtual events */
 static const enum pqos_mon_event os_mon_event[] = {
@@ -59,6 +57,7 @@ static const enum pqos_mon_event os_mon_event[] = {
     PQOS_MON_EVENT_LMEM_BW,
     PQOS_MON_EVENT_TMEM_BW,
     PQOS_PERF_EVENT_LLC_MISS,
+    PQOS_PERF_EVENT_LLC_REF,
     (enum pqos_mon_event)PQOS_PERF_EVENT_CYCLES,
     (enum pqos_mon_event)PQOS_PERF_EVENT_INSTRUCTIONS};
 
@@ -89,8 +88,8 @@ filter(const struct dirent *dir)
  * @retval PQOS_RETVAL_OK on success
  * @retval PQOS_RETVAL_ERROR on error
  */
-static int
-stop_events(struct pqos_mon_data *group)
+PQOS_STATIC int
+os_mon_stop_events(struct pqos_mon_data *group)
 {
         int ret;
         enum pqos_mon_event stopped_evts = (enum pqos_mon_event)0;
@@ -163,8 +162,8 @@ stop_event_error:
  * @retval PQOS_RETVAL_OK on success
  * @retval PQOS_RETVAL_ERROR on error
  */
-static int
-start_events(struct pqos_mon_data *group)
+PQOS_STATIC int
+os_mon_start_events(struct pqos_mon_data *group)
 {
         int ret = PQOS_RETVAL_OK;
         unsigned num_ctrs, i;
@@ -183,7 +182,7 @@ start_events(struct pqos_mon_data *group)
         events = group->event;
         group->intl->perf.event = (enum pqos_mon_event)0;
         group->intl->perf.ctx =
-            malloc(sizeof(group->intl->perf.ctx[0]) * num_ctrs);
+            calloc(num_ctrs, sizeof(group->intl->perf.ctx[0]));
         if (group->intl->perf.ctx == NULL) {
                 LOG_ERROR("Memory allocation failed\n");
                 return PQOS_RETVAL_ERROR;
@@ -257,7 +256,7 @@ start_events(struct pqos_mon_data *group)
 start_event_error:
         /*  Check if all selected events were started */
         if ((group->event & started_evts) != group->event) {
-                stop_events(group);
+                os_mon_stop_events(group);
                 LOG_ERROR("Failed to start all selected "
                           "OS monitoring events\n");
                 free(group->intl->perf.ctx);
@@ -266,76 +265,6 @@ start_event_error:
         return ret;
 }
 
-/**
- * @brief This function polls selected events
- *
- * @param group monitoring structure
- *
- * @return Operation status
- * @retval PQOS_RETVAL_OK on success
- * @retval PQOS_RETVAL_ERROR on error
- */
-static int
-poll_events(struct pqos_mon_data *group)
-{
-        unsigned i;
-        int ret = PQOS_RETVAL_OK;
-
-        if (group->intl->resctrl.event != 0) {
-                ret = resctrl_lock_shared();
-                if (ret != PQOS_RETVAL_OK)
-                        return ret;
-        }
-
-        for (i = 0; i < DIM(os_mon_event); i++) {
-                enum pqos_mon_event evt = os_mon_event[i];
-
-                /**
-                 * poll perf event
-                 */
-                if (group->intl->perf.event & evt) {
-                        ret = perf_mon_poll(group, evt);
-                        if (ret != PQOS_RETVAL_OK)
-                                goto poll_events_exit;
-                }
-
-                /**
-                 * poll resctrl event
-                 */
-                if (group->intl->resctrl.event & evt) {
-                        ret = resctrl_mon_poll(group, evt);
-                        if (ret != PQOS_RETVAL_OK)
-                                goto poll_events_exit;
-                }
-        }
-
-        /**
-         * Calculate values of virtual events
-         */
-        if (group->event & PQOS_MON_EVENT_RMEM_BW) {
-                group->values.mbm_remote_delta = 0;
-                if (group->values.mbm_total_delta >
-                    group->values.mbm_local_delta)
-                        group->values.mbm_remote_delta =
-                            group->values.mbm_total_delta -
-                            group->values.mbm_local_delta;
-        }
-        if (group->event & PQOS_PERF_EVENT_IPC) {
-                if (group->values.ipc_unhalted_delta > 0)
-                        group->values.ipc =
-                            (double)group->values.ipc_retired_delta /
-                            (double)group->values.ipc_unhalted_delta;
-                else
-                        group->values.ipc = 0;
-        }
-
-poll_events_exit:
-        if (group->intl->resctrl.event != 0)
-                resctrl_lock_release();
-
-        return ret;
-}
-
 int
 os_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 {
@@ -354,16 +283,12 @@ os_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
         if (ret != PQOS_RETVAL_OK)
                 return ret;
 
-        m_cpu = cpu;
-
         return ret;
 }
 
 int
 os_mon_fini(void)
 {
-        m_cpu = NULL;
-
         perf_mon_fini();
         resctrl_mon_fini();
 
@@ -387,7 +312,7 @@ os_mon_stop(struct pqos_mon_data *group)
                 return PQOS_RETVAL_PARAM;
 
         /* stop all started events */
-        ret = stop_events(group);
+        ret = os_mon_stop_events(group);
 
         /* free memory */
         if (group->num_cores > 0) {
@@ -398,34 +323,31 @@ os_mon_stop(struct pqos_mon_data *group)
                 free(group->tid_map);
                 group->tid_map = NULL;
         }
-        memset(group, 0, sizeof(*group));
+        if (group->pids != NULL) {
+                free(group->pids);
+                group->pids = NULL;
+        }
 
         return ret;
 }
 
-int
-os_mon_start(const unsigned num_cores,
-             const unsigned *cores,
-             const enum pqos_mon_event event,
-             void *context,
-             struct pqos_mon_data *group)
+/**
+ * @brief Validate if event is listed in capabilities
+ *
+ * @param[in] event list of events
+ *
+ * @return Operational status
+ * @retval PQOS_RETVAL_OK on success
+ * @retval PQOS_RETVAL_PARAM on error
+ */
+PQOS_STATIC int
+os_mon_validate_event(const enum pqos_mon_event event)
 {
-        unsigned i = 0;
-        int ret;
-        const struct pqos_cap *cap;
-        const struct pqos_cpuinfo *cpu;
+        const struct pqos_cap *cap = _pqos_get_cap();
+        unsigned i;
 
-        ASSERT(group != NULL);
-        ASSERT(cores != NULL);
-        ASSERT(num_cores > 0);
-        ASSERT(event > 0);
-
-        _pqos_cap_get(&cap, &cpu);
-
-        /**
-         * Validate if event is listed in capabilities
-         */
         for (i = 0; i < (sizeof(event) * 8); i++) {
+                int ret;
                 const enum pqos_mon_event evt_mask =
                     (enum pqos_mon_event)(1U << i);
                 const struct pqos_monitor *ptr = NULL;
@@ -437,16 +359,53 @@ os_mon_start(const unsigned num_cores,
                 if (ret != PQOS_RETVAL_OK || ptr == NULL)
                         return PQOS_RETVAL_PARAM;
         }
+
+        return PQOS_RETVAL_OK;
+}
+
+int
+os_mon_start_cores(const unsigned num_cores,
+                   const unsigned *cores,
+                   const enum pqos_mon_event event,
+                   void *context,
+                   struct pqos_mon_data *group,
+                   const struct pqos_mon_options *opt)
+{
+        unsigned i = 0;
+        int ret;
+        const struct pqos_cpuinfo *cpu = _pqos_get_cpu();
+
+        ASSERT(group != NULL);
+        ASSERT(cores != NULL);
+        ASSERT(num_cores > 0);
+        ASSERT(event > 0);
+
+        UNUSED_PARAM(opt);
+
+        /* Validate if event is listed in capabilities */
+        ret = os_mon_validate_event(event);
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
+
         /**
          * Check if all requested cores are valid
          * and not used by other monitoring processes.
          */
         for (i = 0; i < num_cores; i++) {
                 const unsigned lcore = cores[i];
+                char name[16];
 
                 ret = pqos_cpu_check_core(cpu, lcore);
                 if (ret != PQOS_RETVAL_OK)
                         return PQOS_RETVAL_PARAM;
+
+                ret = resctrl_mon_assoc_get(lcore, name, sizeof(name));
+                if (ret == PQOS_RETVAL_OK) {
+                        LOG_ERROR("Monitoring on core %u is already started\n",
+                                  lcore);
+                        return PQOS_RETVAL_RESOURCE;
+                } else if (ret != PQOS_RETVAL_RESOURCE)
+                        return ret;
         }
 
         /**
@@ -462,9 +421,11 @@ os_mon_start(const unsigned num_cores,
         for (i = 0; i < num_cores; i++)
                 group->cores[i] = cores[i];
 
-        ret = start_events(group);
-        if (ret != PQOS_RETVAL_OK && group->cores != NULL)
+        ret = os_mon_start_events(group);
+        if (ret != PQOS_RETVAL_OK && group->cores != NULL) {
                 free(group->cores);
+                group->cores = NULL;
+        }
 
         return ret;
 }
@@ -581,8 +542,8 @@ tid_find(const pid_t pid, unsigned *tid_nr, pid_t **tid_map)
  *
  * @retval 1 if PID is correct
  */
-static int
-tid_verify(const pid_t pid)
+PQOS_STATIC int
+os_mon_tid_exists(const pid_t pid)
 {
         int found = 0;
         char buf[64];
@@ -615,13 +576,18 @@ os_mon_start_pids(const unsigned num_pids,
         ASSERT(event > 0);
         ASSERT(pids != NULL);
 
+        /* Validate if event is listed in capabilities */
+        ret = os_mon_validate_event(event);
+        if (ret != PQOS_RETVAL_OK)
+                return ret;
+
         /**
          * Check if all PIDs exists
          */
         for (i = 0; i < num_pids; i++) {
                 pid_t pid = pids[i];
 
-                if (!tid_verify(pid)) {
+                if (!os_mon_tid_exists(pid)) {
                         LOG_ERROR("Task %d does not exist!\n", (int)pid);
                         return PQOS_RETVAL_PARAM;
                 }
@@ -651,11 +617,13 @@ os_mon_start_pids(const unsigned num_pids,
         for (i = 0; i < num_pids; i++)
                 group->pids[i] = pids[i];
 
-        ret = start_events(group);
+        ret = os_mon_start_events(group);
 
 os_mon_start_pids_exit:
-        if (ret != PQOS_RETVAL_OK && tid_map != NULL)
+        if (ret != PQOS_RETVAL_OK && tid_map != NULL) {
                 free(tid_map);
+                group->tid_map = NULL;
+        }
 
         return ret;
 }
@@ -671,6 +639,7 @@ os_mon_add_pids(const unsigned num_pids,
         pid_t *ptr;
         unsigned tid_nr = 0;
         struct pqos_mon_data added;
+        struct pqos_mon_data_internal added_intl;
         struct pqos_mon_perf_ctx *ctx;
         unsigned num_duplicated = 0;
 
@@ -679,6 +648,8 @@ os_mon_add_pids(const unsigned num_pids,
         ASSERT(pids != NULL);
 
         memset(&added, 0, sizeof(added));
+        memset(&added_intl, 0, sizeof(added_intl));
+        added.intl = &added_intl;
 
         /**
          * Check if all PIDs exists
@@ -686,7 +657,7 @@ os_mon_add_pids(const unsigned num_pids,
         for (i = 0; i < num_pids; i++) {
                 pid_t pid = pids[i];
 
-                if (!tid_verify(pid)) {
+                if (!os_mon_tid_exists(pid)) {
                         LOG_ERROR("Task %d does not exist!\n", (int)pid);
                         return PQOS_RETVAL_PARAM;
                 }
@@ -726,10 +697,6 @@ os_mon_add_pids(const unsigned num_pids,
         added.tid_map = tid_map;
         added.event = group->event;
         added.num_pids = num_pids;
-        added.intl = malloc(sizeof(*added.intl));
-        if (added.intl == NULL)
-                goto os_mon_add_pids_exit;
-        memset(added.intl, 0, sizeof(*added.intl));
         if (group->intl->resctrl.mon_group != NULL) {
                 added.intl->resctrl.mon_group =
                     strdup(group->intl->resctrl.mon_group);
@@ -739,7 +706,7 @@ os_mon_add_pids(const unsigned num_pids,
                 }
         }
 
-        ret = start_events(&added);
+        ret = os_mon_start_events(&added);
         if (ret != PQOS_RETVAL_OK)
                 goto os_mon_add_pids_exit;
 
@@ -782,19 +749,17 @@ os_mon_add_pids(const unsigned num_pids,
         }
 
 os_mon_add_pids_exit:
-        if (added.intl != NULL && added.intl->resctrl.mon_group != NULL) {
+        if (added.intl->resctrl.mon_group != NULL) {
                 free(added.intl->resctrl.mon_group);
                 added.intl->resctrl.mon_group = NULL;
         }
         if (ret == PQOS_RETVAL_RESOURCE) {
                 LOG_ERROR("Memory allocation error!\n");
-                stop_events(&added);
-        }
-        if (added.intl != NULL) {
-                if (added.intl->perf.ctx != NULL)
-                        free(added.intl->perf.ctx);
-                free(added.intl);
+                os_mon_stop_events(&added);
         }
+        if (added.intl->perf.ctx != NULL)
+                free(added.intl->perf.ctx);
+
         if (tid_map != NULL)
                 free(tid_map);
         return ret;
@@ -811,6 +776,7 @@ os_mon_remove_pids(const unsigned num_pids,
         pid_t *keep_tid_map = NULL; /* List of not removed TIDs */
         unsigned keep_tid_nr = 0;
         struct pqos_mon_data remove;
+        struct pqos_mon_data_internal remove_intl;
         unsigned removed;
 
         ASSERT(num_pids > 0);
@@ -818,6 +784,8 @@ os_mon_remove_pids(const unsigned num_pids,
         ASSERT(group != NULL);
 
         memset(&remove, 0, sizeof(remove));
+        memset(&remove_intl, 0, sizeof(remove_intl));
+        remove.intl = &remove_intl;
 
         /**
          * Find TID's for not removed tasks
@@ -828,7 +796,7 @@ os_mon_remove_pids(const unsigned num_pids,
                         continue;
 
                 /* pid no longer exists */
-                if (!tid_verify(group->pids[i]))
+                if (!os_mon_tid_exists(group->pids[i]))
                         continue;
 
                 ret = tid_find(group->pids[i], &keep_tid_nr, &keep_tid_map);
@@ -836,11 +804,6 @@ os_mon_remove_pids(const unsigned num_pids,
                         goto os_mon_remove_pids_exit;
         }
 
-        remove.intl =
-            (struct pqos_mon_data_internal *)malloc(sizeof(*remove.intl));
-        if (remove.intl == NULL)
-                goto os_mon_remove_pids_exit;
-        memset(remove.intl, 0, sizeof(*remove.intl));
         remove.intl->perf.event = group->intl->perf.event;
         remove.intl->resctrl.event = group->intl->resctrl.event;
         remove.pids = NULL;
@@ -864,7 +827,7 @@ os_mon_remove_pids(const unsigned num_pids,
                 remove.tid_nr++;
         }
 
-        ret = stop_events(&remove);
+        ret = os_mon_stop_events(&remove);
         if (ret != PQOS_RETVAL_OK)
                 goto os_mon_remove_pids_exit;
 
@@ -904,32 +867,9 @@ os_mon_remove_pids(const unsigned num_pids,
 os_mon_remove_pids_exit:
         if (remove.tid_map != NULL)
                 free(remove.tid_map);
-        if (remove.intl != NULL) {
-                if (remove.intl->perf.ctx)
-                        free(remove.intl->perf.ctx);
-                free(remove.intl);
-        }
+        if (remove.intl->perf.ctx != NULL)
+                free(remove.intl->perf.ctx);
         if (keep_tid_map != NULL)
                 free(keep_tid_map);
         return ret;
 }
-
-int
-os_mon_poll(struct pqos_mon_data **groups, const unsigned num_groups)
-{
-        unsigned i = 0;
-
-        ASSERT(groups != NULL);
-        ASSERT(num_groups > 0);
-
-        for (i = 0; i < num_groups; i++) {
-                int ret = poll_events(groups[i]);
-
-                if (ret != PQOS_RETVAL_OK)
-                        LOG_WARN("Failed to poll event on "
-                                 "group number %u\n",
-                                 i);
-        }
-
-        return PQOS_RETVAL_OK;
-}
