@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2018-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2018-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -57,21 +57,12 @@
 #define OS_MON_EVT_IDX_LLC_MISS 7
 #define OS_MON_EVT_IDX_LLC_REF  8
 
-/**
- * ---------------------------------------
- * Local data structures
- * ---------------------------------------
- */
-static const struct pqos_cpuinfo *m_cpu = NULL;
-
 /**
  * Paths to RDT perf event info
  */
 #ifndef PERF_MON_SUPPORT
 #define PERF_MON_SUPPORT "/proc/sys/kernel/perf_event_paranoid"
 #endif
-static const char *perf_events = "events/";
-static const char *perf_type = "type";
 
 /**
  * Monitoring event type
@@ -204,16 +195,16 @@ set_mon_type(void)
         FILE *fd;
         char file[64], evt[8];
 
-        snprintf(file, sizeof(file) - 1, "%s/%s", PERF_MON_PATH, perf_type);
+        snprintf(file, sizeof(file) - 1, "%s", PERF_MON_TYPE);
         fd = pqos_fopen(file, "r");
         if (fd == NULL)
                 return PQOS_RETVAL_RESOURCE;
         if (fgets(evt, sizeof(evt), fd) == NULL) {
                 LOG_ERROR("Failed to read perf monitoring type!\n");
-                fclose(fd);
+                pqos_fclose(fd);
                 return PQOS_RETVAL_ERROR;
         }
-        fclose(fd);
+        pqos_fclose(fd);
 
         os_mon_type = (int)strtol(evt, NULL, 0);
         if (os_mon_type == 0) {
@@ -297,8 +288,7 @@ set_rdt_event_attrs(const int idx, const char *fname)
         /**
          * Read event type from file system
          */
-        snprintf(file, sizeof(file) - 1, "%s/%s%s", PERF_MON_PATH, perf_events,
-                 fname);
+        snprintf(file, sizeof(file) - 1, "%s/%s", PERF_MON_EVENTS, fname);
         fd = pqos_fopen(file, "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to open %s!\n", file);
@@ -306,10 +296,10 @@ set_rdt_event_attrs(const int idx, const char *fname)
         }
         if (fgets(buf, sizeof(buf), fd) == NULL) {
                 LOG_ERROR("Failed to read OS monitoring event!\n");
-                fclose(fd);
+                pqos_fclose(fd);
                 return PQOS_RETVAL_ERROR;
         }
-        fclose(fd);
+        pqos_fclose(fd);
         p = buf;
         strsep(&p, "=");
         if (p == NULL) {
@@ -322,15 +312,14 @@ set_rdt_event_attrs(const int idx, const char *fname)
         /**
          * Read scale factor from file system
          */
-        snprintf(file, sizeof(file) - 1, "%s/%s%s.scale", PERF_MON_PATH,
-                 perf_events, fname);
+        snprintf(file, sizeof(file) - 1, "%s/%s.scale", PERF_MON_EVENTS, fname);
         fd = pqos_fopen(file, "r");
         if (fd == NULL) {
                 LOG_ERROR("Failed to open OS monitoring event scale file!\n");
                 return PQOS_RETVAL_ERROR;
         }
         ret = fscanf(fd, "%10lf", &sf);
-        fclose(fd);
+        pqos_fclose(fd);
         if (ret < 1) {
                 LOG_ERROR("Failed to read OS monitoring event scale factor!\n");
                 return PQOS_RETVAL_ERROR;
@@ -368,7 +357,7 @@ set_mon_events(void)
         /**
          * Read and store event data in table
          */
-        snprintf(dir, sizeof(dir) - 1, "%s/%s", PERF_MON_PATH, perf_events);
+        snprintf(dir, sizeof(dir) - 1, "%s", PERF_MON_EVENTS);
         files = scandir(dir, &namelist, filter, NULL);
         if (files <= 0) {
                 LOG_ERROR("Failed to read perf monitoring events directory!\n");
@@ -427,16 +416,16 @@ perf_mon_init(const struct pqos_cpuinfo *cpu, const struct pqos_cap *cap)
 {
         int ret;
         unsigned i;
-        struct stat st;
 
         ASSERT(cpu != NULL);
 
+        UNUSED_PARAM(cpu);
         UNUSED_PARAM(cap);
 
         /**
          * Perf monitoring not supported
          */
-        if (stat(PERF_MON_SUPPORT, &st) != 0) {
+        if (!pqos_file_exists(PERF_MON_SUPPORT)) {
                 LOG_INFO("Perf monitoring not supported.");
                 return PQOS_RETVAL_RESOURCE;
 
@@ -472,15 +461,17 @@ perf_mon_init_exit:
                          events_tab[i].desc);
         }
 
-        m_cpu = cpu;
-
         return ret;
 }
 
 int
 perf_mon_fini(void)
 {
-        m_cpu = NULL;
+        unsigned i;
+
+        all_evt_mask = (enum pqos_mon_event)0;
+        for (i = 0; i < DIM(events_tab); ++i)
+                events_tab[i].supported = 0;
 
         return PQOS_RETVAL_OK;
 }
