@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2014-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2014-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -50,13 +50,19 @@
 extern "C" {
 #endif
 
+#ifndef SWIG
+#define PQOS_DEPRECATED __attribute__((deprecated))
+#else
+#define PQOS_DEPRECATED
+#endif
+
 /*
  * =======================================
  * Various defines
  * =======================================
  */
 
-#define PQOS_VERSION      40401 /**< version 4.4.1 */
+#define PQOS_VERSION      40601 /**< version 4.6.1 */
 #define PQOS_MAX_COS      16    /**< 16 x COS */
 #define PQOS_MAX_L3CA_COS PQOS_MAX_COS
 #define PQOS_MAX_L2CA_COS PQOS_MAX_COS
@@ -95,40 +101,27 @@ enum pqos_interface {
  * =======================================
  */
 
+#if PQOS_VERSION >= 50000
+enum pqos_cdp_config {
+        PQOS_REQUIRE_CDP_ANY = 0, /**< app will work with any CDP
+                                     setting */
+        PQOS_REQUIRE_CDP_OFF,     /**< app not compatible with CDP */
+        PQOS_REQUIRE_CDP_ON       /**< app requires CDP */
+};
+#else
 enum pqos_cdp_config {
         PQOS_REQUIRE_CDP_OFF = 0, /**< app not compatible with CDP */
         PQOS_REQUIRE_CDP_ON,      /**< app requires CDP */
         PQOS_REQUIRE_CDP_ANY      /**< app will work with any CDP
                                      setting */
 };
+#endif
 
 /**
  * Resource Monitoring ID (RMID) definition
  */
 typedef uint32_t pqos_rmid_t;
 
-#ifdef PQOS_RMID_CUSTOM
-/**
- * RMID initialization types
- */
-enum pqos_rmid_type {
-        PQOS_RMID_TYPE_DEFAULT = 0, /**< Default sequential init */
-        PQOS_RMID_TYPE_MAP          /**< Custom Core to RMID mapping */
-};
-
-/**
- * RMID configuration
- */
-struct pqos_rmid_config {
-        enum pqos_rmid_type type; /**< rmid initialization type */
-        struct {
-                unsigned num;      /**< number of rmid to core mappings */
-                pqos_rmid_t *rmid; /**< rmid table */
-                unsigned *core;    /**< core table */
-        } map;
-};
-#endif
-
 /**
  * PQoS library configuration structure
  *
@@ -162,9 +155,6 @@ struct pqos_config {
         void *context_log;
         int verbose;
         enum pqos_interface interface;
-#ifdef PQOS_RMID_CUSTOM
-        struct pqos_rmid_config rmid_cfg;
-#endif
 };
 
 /**
@@ -216,36 +206,37 @@ enum pqos_cap_type {
  * L3 Cache Allocation (CA) capability structure
  */
 struct pqos_cap_l3ca {
-        unsigned mem_size;       /**< byte size of the structure */
-        unsigned num_classes;    /**< number of classes of service */
-        unsigned num_ways;       /**< number of cache ways */
-        unsigned way_size;       /**< way size in bytes */
-        uint64_t way_contention; /**< ways contention bit mask */
-        int cdp;                 /**< code data prioritization feature
-                                    support */
-        int cdp_on;              /**< code data prioritization on or
-                                    off */
+        unsigned mem_size;           /**< byte size of the structure */
+        unsigned num_classes;        /**< number of classes of service */
+        unsigned num_ways;           /**< number of cache ways */
+        unsigned way_size;           /**< way size in bytes */
+        uint64_t way_contention;     /**< ways contention bit mask */
+        int cdp;                     /**< code data prioritization feature
+                                        support */
+        int cdp_on;                  /**< code data prioritization on or off */
+        unsigned non_contiguous_cbm; /**< Non-Contiguous CBM support */
 };
 
 /**
  * L2 Cache Allocation (CA) capability structure
  */
 struct pqos_cap_l2ca {
-        unsigned mem_size;       /**< byte size of the structure */
-        unsigned num_classes;    /**< number of classes of service */
-        unsigned num_ways;       /**< number of cache ways */
-        unsigned way_size;       /**< way size in bytes */
-        uint64_t way_contention; /**< ways contention bit mask */
-        int cdp;                 /**< code data prioritization feature
-                                    support */
-        int cdp_on;              /**< code data prioritization on or off */
+        unsigned mem_size;           /**< byte size of the structure */
+        unsigned num_classes;        /**< number of classes of service */
+        unsigned num_ways;           /**< number of cache ways */
+        unsigned way_size;           /**< way size in bytes */
+        uint64_t way_contention;     /**< ways contention bit mask */
+        int cdp;                     /**< code data prioritization feature
+                                        support */
+        int cdp_on;                  /**< code data prioritization on or off */
+        unsigned non_contiguous_cbm; /**< Non-Contiguous CBM support */
 };
 
 /**
  * Memory Bandwidth Allocation configuration enumeration
  */
 enum pqos_mba_config {
-        PQOS_MBA_ANY,     /**< currently enabled configuration */
+        PQOS_MBA_ANY = 0, /**< currently enabled configuration */
         PQOS_MBA_DEFAULT, /**< direct MBA hardware configuration
                              (percentage) */
         PQOS_MBA_CTRL     /**< MBA controller configuration (MBps) */
@@ -351,6 +342,9 @@ struct pqos_coreinfo {
         unsigned l2_id;    /**< L2 cluster id */
         unsigned l3cat_id; /**< L3 CAT classes id */
         unsigned mba_id;   /**< MBA id */
+#if (PQOS_VERSION >= 50000 || defined PQOS_SNC)
+        unsigned numa; /**< numa node in the system */
+#endif
 };
 
 /**
@@ -512,15 +506,49 @@ int pqos_mon_assoc_get(const unsigned lcore, pqos_rmid_t *rmid);
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
  *
+ * @deprecated since 5.0.0
+ * @see pqos_mon_start_cores()
+ *
  * @note As of Kernel 4.10, Intel(R) RDT perf results per core are found to
  *       be incorrect.
  */
+#if PQOS_VERSION >= 50000
+PQOS_DEPRECATED
+#endif
 int pqos_mon_start(const unsigned num_cores,
                    const unsigned *cores,
                    const enum pqos_mon_event event,
                    void *context,
                    struct pqos_mon_data *group);
 
+/**
+ * @brief Starts resource monitoring on selected group of cores
+ *
+ * The function sets up content of the \a group structure.
+ *
+ * Note that \a event cannot select PQOS_PERF_EVENT_IPC or
+ * PQOS_PERF_EVENT_L3_MISS events without any PQoS event
+ * selected at the same time.
+ *
+ * @param [in] num_cores number of cores in \a cores array
+ * @param [in] cores array of logical core id's
+ * @param [in] event combination of monitoring events
+ * @param [in] context a pointer for application's convenience
+ *            (unused by the library)
+ * @param [out] group a pointer to monitoring structure
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ *
+ * @note As of Kernel 4.10, Intel(R) RDT perf results per core are found to
+ *       be incorrect.
+ */
+int pqos_mon_start_cores(const unsigned num_cores,
+                         const unsigned *cores,
+                         const enum pqos_mon_event event,
+                         void *context,
+                         struct pqos_mon_data **group);
+
 /**
  * @brief Starts resource monitoring of selected \a pid (process)
  *
@@ -530,9 +558,15 @@ int pqos_mon_start(const unsigned num_cores,
  *             (unused by the library)
  * @param [in,out] group a pointer to monitoring structure
  *
+ * @deprecated since 5.0.0
+ * @see pqos_mon_start_pids2()
+ *
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
  */
+#if PQOS_VERSION >= 50000
+PQOS_DEPRECATED
+#endif
 int pqos_mon_start_pid(const pid_t pid,
                        const enum pqos_mon_event event,
                        void *context,
@@ -548,15 +582,40 @@ int pqos_mon_start_pid(const pid_t pid,
  *             (unused by the library)
  * @param [in,out] group a pointer to monitoring structure
  *
+ * @deprecated since 5.0.0
+ * @see pqos_mon_start_pids2()
+ *
  * @return Operations status
  * @retval PQOS_RETVAL_OK on success
  */
+#if PQOS_VERSION >= 50000
+PQOS_DEPRECATED
+#endif
 int pqos_mon_start_pids(const unsigned num_pids,
                         const pid_t *pids,
                         const enum pqos_mon_event event,
                         void *context,
                         struct pqos_mon_data *group);
 
+/**
+ * @brief Starts resource monitoring of selected \a pids (processes)
+ *
+ * @param [in] num_pids number of pids in \a pids array
+ * @param [in] pids array of process ID
+ * @param [in] event monitoring event id
+ * @param [in] context a pointer for application's convenience
+ *             (unused by the library)
+ * @param [out] group a pointer to monitoring structure
+ *
+ * @return Operations status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_mon_start_pids2(const unsigned num_pids,
+                         const pid_t *pids,
+                         const enum pqos_mon_event event,
+                         void *context,
+                         struct pqos_mon_data **group);
+
 /**
  * @brief Adds pids to the resource monitoring grpup
  *
@@ -761,13 +820,46 @@ int pqos_alloc_release_pid(const pid_t *task_array, const unsigned task_num);
  * @param [in] l2_cdp_cfg requested L2 CAT CDP config
  * @param [in] mba_cfg requested MBA config
  *
+ * @deprecated since 5.0.0
+ * @see pqos_alloc_reset_config()
+ *
  * @return Operation status
  * @retval PQOS_RETVAL_OK on success
  */
+#if PQOS_VERSION >= 50000
+PQOS_DEPRECATED
+#endif
 int pqos_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
                      const enum pqos_cdp_config l2_cdp_cfg,
                      const enum pqos_mba_config mba_cfg);
 
+/**
+ * Configuration of allocation
+ */
+struct pqos_alloc_config {
+        enum pqos_cdp_config l3_cdp; /**< requested L3 CAT CDP config */
+        enum pqos_cdp_config l2_cdp; /**< requested L2 CAT CDP config */
+        enum pqos_mba_config mba;    /**< requested MBA config */
+        int reserved[5];             /**< reserved for future use */
+};
+
+/**
+ * @brief Resets configuration of allocation technologies
+ *
+ * Reverts CAT/MBA state to the one after reset:
+ * - all cores associated with COS0
+ * - all COS are set to give access to entire resource
+ *
+ * As part of allocation reset CDP, MBA can be performed.
+ * This can be requested via \a cfg.
+ *
+ * @param [in] cfg requested configuration
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_alloc_reset_config(const struct pqos_alloc_config *cfg);
+
 /*
  * =======================================
  * L3 cache allocation
@@ -987,6 +1079,19 @@ unsigned *pqos_cpu_get_l3cat_ids(const struct pqos_cpuinfo *cpu,
  */
 unsigned *pqos_cpu_get_sockets(const struct pqos_cpuinfo *cpu, unsigned *count);
 
+#if (PQOS_VERSION >= 50000 || defined PQOS_SNC)
+/**
+ * @brief Retrieves numa id's from cpu info structure
+ *
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
+ * @param [out] count place to store actual number of sockets returned
+ *
+ * @return Allocated array of size \a count populated with numa id's
+ * @retval NULL on error
+ */
+unsigned *pqos_cpu_get_numa(const struct pqos_cpuinfo *cpu, unsigned *count);
+#endif
+
 /**
  * @brief Retrieves L2 id's from cpu info structure
  *
@@ -1029,20 +1134,6 @@ unsigned *pqos_cpu_get_cores(const struct pqos_cpuinfo *cpu,
                              const unsigned socket,
                              unsigned *count);
 
-/**
- * @brief Retrieves core id's from cpu info structure for \a l3cat_id
- *
- * @param [in] cpu CPU information structure from \a pqos_cap_get
- * @param [in] l3cat_id to enumerate
- * @param [out] count place to store actual number of core id's returned
- *
- * @return Allocated core id array
- * @retval NULL on error
- */
-unsigned *pqos_cpu_get_cores_l3cat_id(const struct pqos_cpuinfo *cpu,
-                                      const unsigned l3cat_id,
-                                      unsigned *count);
-
 /**
  * @brief Retrieves task id's from resctrl task file for a given COS
  *
@@ -1058,7 +1149,7 @@ unsigned *pqos_pid_get_pid_assoc(const unsigned class_id, unsigned *count);
  * @brief Retrieves one core id from cpu info structure for \a mba_id
  *
  * @param [in] cpu CPU information structure from \a pqos_cap_get
- * @param [in] mba to enumerate
+ * @param [in] mba_id to enumerate
  * @param [out] lcore place to store returned core id
  *
  * @return Operation status
@@ -1093,11 +1184,27 @@ int pqos_cpu_get_one_core(const struct pqos_cpuinfo *cpu,
                           const unsigned socket,
                           unsigned *lcore);
 
+#if (PQOS_VERSION >= 50000 || defined PQOS_SNC)
+/**
+ * @brief Retrieves one core id from cpu info structure for \a numaid
+ *
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
+ * @param [in] numaid id to enumerate
+ * @param [out] lcore place to store returned core id
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_cpu_get_one_by_numaid(const struct pqos_cpuinfo *cpu,
+                               const unsigned numaid,
+                               unsigned *lcore);
+#endif
+
 /**
  * @brief Retrieves one core id from cpu info structure for \a l3cat id
  *
  * @param [in] cpu CPU information structure from \a pqos_cap_get
- * @param [in] l3cat id to enumerate
+ * @param [in] l3cat_id id to enumerate
  * @param [out] lcore place to store returned core id
  *
  * @return Operation status
@@ -1145,6 +1252,22 @@ int pqos_cpu_get_socketid(const struct pqos_cpuinfo *cpu,
                           const unsigned lcore,
                           unsigned *socket);
 
+#if (PQOS_VERSION >= 50000 || defined PQOS_SNC)
+/**
+ * @brief Retrieves NUMA cluster id for given logical core id
+ *
+ * @param [in] cpu CPU information structure from \a pqos_cap_get
+ * @param [in] lcore logical core id
+ * @param [out] numa location to store numa id at
+ *
+ * @return Operation status
+ * @retval PQOS_RETVAL_OK on success
+ */
+int pqos_cpu_get_numaid(const struct pqos_cpuinfo *cpu,
+                        const unsigned lcore,
+                        unsigned *numa);
+#endif
+
 /**
  * @brief Retrieves monitoring cluster id for given logical core id
  *
@@ -1367,6 +1490,23 @@ int pqos_mon_get_value(const struct pqos_mon_data *const group,
  */
 int pqos_mon_get_ipc(const struct pqos_mon_data *const group, double *value);
 
+/**
+ * @brief Frees memory previously allocated and returned by the library
+ * functions.
+ *
+ * Some pqos functions return memory buffers to their callers.
+ * It's the responsibility of the caller to free this memory when no longer
+ * needed.
+ *
+ * Call this function from Foreign Function Interfaces to avoid memory leaks.
+ *
+ * @param [in] ptr to memory area allocated and returned by pqos functions.
+ *             When NULL, the function does nothing.
+ *
+ * @return Void.
+ */
+void pqos_free(void *ptr);
+
 #ifdef __cplusplus
 }
 #endif
