@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,6 +42,7 @@
 extern "C" {
 #endif
 
+#include "pqos.h"
 #include "types.h"
 
 /**
@@ -54,6 +55,16 @@ extern "C" {
  */
 PQOS_LOCAL int _pqos_utils_init(int interface);
 
+/**
+ * @brief Internal API to retrieve \a type of capability
+ *
+ * @param [in] type capability type to look for
+ *
+ * @return Pointer to selected capability
+ */
+PQOS_LOCAL const struct pqos_capability *
+_pqos_cap_get_type(const enum pqos_cap_type type);
+
 #ifdef __cplusplus
 }
 #endif
