@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2017-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2017-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,6 +34,13 @@
  * @brief Platform QoS utility - capability module
  *
  */
+
+#include "cap.h"
+
+#include "common.h"
+#include "main.h"
+#include "pqos.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -41,10 +48,6 @@
 #ifdef __linux__
 #include <sys/utsname.h>
 #endif
-#include "pqos.h"
-
-#include "main.h"
-#include "cap.h"
 
 #define BUFFER_SIZE 512
 #define NON_VERBOSE 0
@@ -107,8 +110,18 @@ get_mon_event_name(int event)
                 return "Remote Memory Bandwidth (RMEM) (calculated)";
         case PQOS_PERF_EVENT_LLC_MISS:
                 return "LLC misses";
+        case PQOS_PERF_EVENT_LLC_REF:
+                return "LLC references";
         case PQOS_PERF_EVENT_IPC:
                 return "Instructions/Clock (IPC)";
+        case PQOS_PERF_EVENT_LLC_MISS_PCIE_READ:
+                return "LLC misses - pcie read";
+        case PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE:
+                return "LLC misses - pcie write";
+        case PQOS_PERF_EVENT_LLC_REF_PCIE_READ:
+                return "LLC references - pcie read";
+        case PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE:
+                return "LLC references - pcie write";
         default:
                 return "unknown";
         }
@@ -153,7 +166,12 @@ cap_print_features_mon(const unsigned indent,
                         break;
 
                 case PQOS_PERF_EVENT_LLC_MISS:
+                case PQOS_PERF_EVENT_LLC_REF:
                 case PQOS_PERF_EVENT_IPC:
+                case PQOS_PERF_EVENT_LLC_REF_PCIE_READ:
+                case PQOS_PERF_EVENT_LLC_MISS_PCIE_READ:
+                case PQOS_PERF_EVENT_LLC_REF_PCIE_WRITE:
+                case PQOS_PERF_EVENT_LLC_MISS_PCIE_WRITE:
                         buffer = buffer_other;
                         break;
 
@@ -228,6 +246,8 @@ cap_print_features_l3ca(const unsigned indent,
         printf_indent(indent + 4, "CDP: %s\n",
                       l3ca->cdp ? (l3ca->cdp_on ? "enabled" : "disabled")
                                 : "unsupported");
+        printf_indent(indent + 4, "Non-Contiguous CBM: %s\n",
+                      l3ca->non_contiguous_cbm ? "supported" : "unsupported");
         printf_indent(indent + 4, "Num COS: %u\n", l3ca->num_classes);
 
         if (!verbose)
@@ -264,6 +284,8 @@ cap_print_features_l2ca(const unsigned indent,
         printf_indent(indent + 4, "CDP: %s\n",
                       l2ca->cdp ? (l2ca->cdp_on ? "enabled" : "disabled")
                                 : "unsupported");
+        printf_indent(indent + 4, "Non-Contiguous CBM: %s\n",
+                      l2ca->non_contiguous_cbm ? "supported" : "unsupported");
         printf_indent(indent + 4, "Num COS: %u\n", l2ca->num_classes);
 
         if (!verbose)
