@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -199,6 +199,24 @@ test_interface_init(struct test_data *data)
         return 0;
 }
 
+static inline int
+test_fini(void **state)
+{
+        struct test_data *data = (struct test_data *)*state;
+
+        if (data != NULL) {
+                if (data->cpu != NULL)
+                        free(data->cpu);
+                if (data->cap != NULL)
+                        free(data->cap);
+                if (data->cap_mon != NULL)
+                        free(data->cap_mon);
+                free(data);
+        }
+
+        return 0;
+}
+
 static inline int
 test_init(void **state, unsigned technology)
 {
@@ -208,25 +226,29 @@ test_init(void **state, unsigned technology)
         data = calloc(1, sizeof(struct test_data));
         if (data == NULL)
                 return -1;
-
-        ret = test_cpuinfo_init(data);
-        if (ret != 0)
-                return ret;
-
-        ret = test_cap_init(data, technology);
-        if (ret != 0)
-                return ret;
-
-        ret = test_config_init(data);
-        if (ret != 0)
-                return ret;
-
-        ret = test_interface_init(data);
-        if (ret != 0)
-                return ret;
-
         *state = data;
 
+        do {
+                ret = test_cpuinfo_init(data);
+                if (ret != 0)
+                        break;
+
+                ret = test_cap_init(data, technology);
+                if (ret != 0)
+                        break;
+
+                ret = test_config_init(data);
+                if (ret != 0)
+                        break;
+
+                ret = test_interface_init(data);
+                if (ret != 0)
+                        break;
+        } while (0);
+
+        if (ret != 0)
+                test_fini(state);
+
         return ret;
 }
 
@@ -248,6 +270,12 @@ test_init_mba(void **state)
         return test_init(state, 1 << PQOS_CAP_TYPE_MBA);
 }
 
+static inline int
+test_init_mon(void **state)
+{
+        return test_init(state, 1 << PQOS_CAP_TYPE_MON);
+}
+
 static inline int
 test_init_all(void **state)
 {
@@ -256,6 +284,7 @@ test_init_all(void **state)
         technology |= 1 << PQOS_CAP_TYPE_MBA;
         technology |= 1 << PQOS_CAP_TYPE_L3CA;
         technology |= 1 << PQOS_CAP_TYPE_L2CA;
+        technology |= 1 << PQOS_CAP_TYPE_MON;
 
         return test_init(state, technology);
 }
@@ -265,21 +294,3 @@ test_init_unsupported(void **state)
 {
         return test_init(state, 0);
 }
-
-static inline int
-test_fini(void **state)
-{
-        struct test_data *data = (struct test_data *)*state;
-
-        if (data != NULL) {
-                if (data->cpu != NULL)
-                        free(data->cpu);
-                if (data->cap != NULL)
-                        free(data->cap);
-                if (data->cap_mon != NULL)
-                        free(data->cap_mon);
-                free(data);
-        }
-
-        return 0;
-}
