@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,16 +30,8 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
-
 #include "allocation.h"
+#include "test.h"
 
 /* ======== mock ======== */
 
@@ -60,6 +52,7 @@ hw_alloc_assoc_unused(const unsigned technology,
                       unsigned *class_id)
 {
         int ret;
+
         check_expected(technology);
         check_expected(l3cat_id);
         check_expected(l2cat_id);
@@ -83,7 +76,9 @@ hw_alloc_assign_l2ca(void **state)
         unsigned core_array[] = {3};
         unsigned core_num = 1;
         unsigned class_id;
-        will_return(__wrap__pqos_cap_get, data->cpu);
+
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(hw_alloc_assoc_unused, technology, technology);
         expect_value(hw_alloc_assoc_unused, l3cat_id, 0);
@@ -110,7 +105,9 @@ hw_alloc_assign_l3ca(void **state)
         unsigned core_array[] = {3};
         unsigned core_num = 1;
         unsigned class_id;
-        will_return(__wrap__pqos_cap_get, data->cpu);
+
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(hw_alloc_assoc_unused, technology, technology);
         expect_value(hw_alloc_assoc_unused, l3cat_id, 0);
@@ -137,7 +134,9 @@ hw_alloc_assign_mba(void **state)
         unsigned core_array[] = {5};
         unsigned core_num = 1;
         unsigned class_id;
-        will_return(__wrap__pqos_cap_get, data->cpu);
+
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(hw_alloc_assoc_unused, technology, technology);
         expect_value(hw_alloc_assoc_unused, l3cat_id, 0);
