@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,17 +30,9 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
-#include "mock_cap.h"
-
 #include "allocation.h"
+#include "mock_cap.h"
+#include "test.h"
 
 /* ======== mock ======== */
 
@@ -81,8 +73,8 @@ test_hw_alloc_assoc_unused_l2ca(void **state)
         unsigned technology = 1 << PQOS_CAP_TYPE_L2CA;
         unsigned class_id;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return_always(hw_alloc_assoc_read, PQOS_RETVAL_OK);
 
@@ -90,9 +82,6 @@ test_hw_alloc_assoc_unused_l2ca(void **state)
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(class_id, 2);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = hw_alloc_assoc_unused(technology, 0, 2, 0, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(class_id, 1);
@@ -106,17 +95,14 @@ test_hw_alloc_assoc_unused_l3ca(void **state)
         unsigned technology = 1 << PQOS_CAP_TYPE_L3CA;
         unsigned class_id;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return_always(hw_alloc_assoc_read, PQOS_RETVAL_OK);
 
         ret = hw_alloc_assoc_unused(technology, 0, 0, 0, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = hw_alloc_assoc_unused(technology, 1, 0, 0, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(class_id, 3);
@@ -130,17 +116,14 @@ test_hw_alloc_assoc_unused_mba(void **state)
         unsigned technology = 1 << PQOS_CAP_TYPE_MBA;
         unsigned class_id;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return_always(hw_alloc_assoc_read, PQOS_RETVAL_OK);
 
         ret = hw_alloc_assoc_unused(technology, 0, 0, 0, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = hw_alloc_assoc_unused(technology, 0, 0, 1, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(class_id, 3);
@@ -154,17 +137,14 @@ test_hw_alloc_assoc_unused_l2ca_when_l3_present(void **state)
         unsigned technology = 1 << PQOS_CAP_TYPE_L2CA;
         unsigned class_id;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return_always(hw_alloc_assoc_read, PQOS_RETVAL_OK);
 
         ret = hw_alloc_assoc_unused(technology, 0, 0, 0, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = hw_alloc_assoc_unused(technology, 0, 3, 0, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(class_id, 1);
@@ -193,4 +173,4 @@ main(void)
         result += cmocka_run_group_tests(tests_all, test_init_all, test_fini);
 
         return result;
-}
\ No newline at end of file
+}
