@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,16 +30,8 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
-
 #include "allocation.h"
+#include "test.h"
 
 /* ======== mock ======== */
 
@@ -61,7 +53,8 @@ test_hw_alloc_reset_assoc(void **state)
         int ret;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         for (i = 0; i < data->cpu->num_cores; i++) {
                 unsigned lcore = data->cpu->cores[i].lcore;
@@ -82,7 +75,8 @@ test_hw_alloc_reset_assoc_error(void **state)
         int ret;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         for (i = 0; i < data->cpu->num_cores; i++) {
                 unsigned lcore = data->cpu->cores[i].lcore;
@@ -108,4 +102,4 @@ main(void)
         result += cmocka_run_group_tests(tests, test_init_l3ca, test_fini);
 
         return result;
-}
\ No newline at end of file
+}
