@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,20 +30,16 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
-
 #include "hw_cap.h"
 #include "machine.h"
+#include "test.h"
 
 #define MAX_CPUID_LEAFS 20
 
+#define NON_CONTIGUOUS_CBM_UNSUPPORTED 0
+#define NON_CONTIGUOUS_CBM_SUPPORTED   1
+#define NON_CONTIGUOUS_CBM_BIT         3
+
 struct test_lcpuid {
         unsigned leaf;
         unsigned subleaf;
@@ -343,7 +339,7 @@ test_hw_cap_mon_discover_llc_miss(void **state)
         ret = hw_cap_mon_discover(&cap_mon, data->cpu);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_non_null(cap_mon);
-        assert_int_equal(cap_mon->num_events, 2);
+        assert_int_equal(cap_mon->num_events, 3);
         assert_int_equal(cap_mon->events[1].type, PQOS_PERF_EVENT_LLC_MISS);
         assert_int_equal(cap_mon->events[1].max_rmid, 0);
         assert_int_equal(cap_mon->events[1].scale_factor, 0);
@@ -352,6 +348,34 @@ test_hw_cap_mon_discover_llc_miss(void **state)
         free(cap_mon);
 }
 
+static void
+test_hw_cap_mon_discover_llc_ref(void **state)
+{
+        struct test_data *data = (struct test_data *)*state;
+        struct pqos_cap_mon *cap_mon = NULL;
+        int ret;
+        uint32_t max_rmid = 10;
+        uint32_t scale_factor = 128;
+        uint32_t counter_length = 24;
+
+        _lcpuid_add(0x7, 0x0, 0x0, 0x1000, 0x0, 0x0);
+        _lcpuid_add(0xf, 0x0, 0x0, max_rmid - 1, 0x0, 0x2);
+        _lcpuid_add(0xf, 0x1, counter_length - 24, scale_factor, max_rmid - 1,
+                    0x4);
+        _lcpuid_add(0xa, 0x0, 0x7300803, 0x0, 0x0, 0x0);
+
+        ret = hw_cap_mon_discover(&cap_mon, data->cpu);
+        assert_int_equal(ret, PQOS_RETVAL_OK);
+        assert_non_null(cap_mon);
+        assert_int_equal(cap_mon->num_events, 3);
+        assert_int_equal(cap_mon->events[2].type, PQOS_PERF_EVENT_LLC_REF);
+        assert_int_equal(cap_mon->events[2].max_rmid, 0);
+        assert_int_equal(cap_mon->events[2].scale_factor, 0);
+        assert_int_equal(cap_mon->events[2].counter_length, 0);
+
+        free(cap_mon);
+}
+
 /* ======== test_hw_cap_l3ca_discover ======== */
 
 static void
@@ -396,6 +420,8 @@ test_hw_cap_l3ca_discover(void **state)
         assert_int_equal(cap_l3ca.cdp, 0);
         assert_int_equal(cap_l3ca.cdp_on, 0);
         assert_int_equal(cap_l3ca.way_contention, way_contention);
+        assert_int_equal(cap_l3ca.non_contiguous_cbm,
+                         NON_CONTIGUOUS_CBM_UNSUPPORTED);
 }
 
 static void
@@ -444,6 +470,29 @@ test_hw_cap_l3ca_discover_cdp(void **state)
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
 
+static void
+test_hw_cap_l3ca_discover_non_contiguous_cbm(void **state)
+{
+        struct test_data *data = (struct test_data *)*state;
+        struct pqos_cap_l3ca cap_l3ca;
+        int ret;
+        uint32_t num_classes = 16;
+        uint32_t num_ways = 11;
+        uint32_t way_contention = 0x600;
+        uint32_t non_contiguous_cbm = NON_CONTIGUOUS_CBM_SUPPORTED
+                                      << NON_CONTIGUOUS_CBM_BIT;
+
+        _lcpuid_add(0x07, 0x0, 0x0, 0x8000, 0x0, 0x0);
+        _lcpuid_add(0x10, 0x0, 0x0, 0x2, 0x0, 0x0);
+        _lcpuid_add(0x10, 0x1, num_ways - 1, way_contention, non_contiguous_cbm,
+                    num_classes - 1);
+
+        ret = hw_cap_l3ca_discover(&cap_l3ca, data->cpu);
+        assert_int_equal(ret, PQOS_RETVAL_OK);
+        assert_int_equal(cap_l3ca.non_contiguous_cbm,
+                         NON_CONTIGUOUS_CBM_SUPPORTED);
+}
+
 /* ======== test_hw_cap_l2ca_discover ======== */
 
 static void
@@ -495,6 +544,8 @@ test_hw_cap_l2ca_discover(void **state)
         assert_int_equal(cap_l2ca.cdp, 0);
         assert_int_equal(cap_l2ca.cdp_on, 0);
         assert_int_equal(cap_l2ca.way_contention, way_contention);
+        assert_int_equal(cap_l2ca.non_contiguous_cbm,
+                         NON_CONTIGUOUS_CBM_UNSUPPORTED);
 }
 
 static void
@@ -543,6 +594,29 @@ test_hw_cap_l2ca_discover_cdp(void **state)
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
 
+static void
+test_hw_cap_l2ca_discover_non_contiguous_cbm(void **state)
+{
+        struct test_data *data = (struct test_data *)*state;
+        struct pqos_cap_l2ca cap_l2ca;
+        int ret;
+        uint32_t num_classes = 16;
+        uint32_t num_ways = 11;
+        uint32_t way_contention = 0x600;
+        uint32_t non_contiguous_cbm = NON_CONTIGUOUS_CBM_SUPPORTED
+                                      << NON_CONTIGUOUS_CBM_BIT;
+
+        _lcpuid_add(0x07, 0x0, 0x0, 0x8000, 0x0, 0x0);
+        _lcpuid_add(0x10, 0x0, 0x0, 0x4, 0x0, 0x0);
+        _lcpuid_add(0x10, 0x2, num_ways - 1, way_contention, non_contiguous_cbm,
+                    num_classes - 1);
+
+        ret = hw_cap_l2ca_discover(&cap_l2ca, data->cpu);
+        assert_int_equal(ret, PQOS_RETVAL_OK);
+        assert_int_equal(cap_l2ca.non_contiguous_cbm,
+                         NON_CONTIGUOUS_CBM_SUPPORTED);
+}
+
 /* ======== hw_cap_mba_discover ======== */
 
 static void
@@ -626,16 +700,21 @@ main(void)
             cmocka_unit_test_setup(test_hw_cap_mon_discover_rmem, _init),
             cmocka_unit_test_setup(test_hw_cap_mon_discover_ipc, _init),
             cmocka_unit_test_setup(test_hw_cap_mon_discover_llc_miss, _init),
+            cmocka_unit_test_setup(test_hw_cap_mon_discover_llc_ref, _init),
             cmocka_unit_test_setup(test_hw_cap_l3ca_discover_unsupported,
                                    _init),
             cmocka_unit_test_setup(test_hw_cap_l3ca_discover, _init),
             cmocka_unit_test_setup(test_hw_cap_l3ca_discover_cdp, _init),
+            cmocka_unit_test_setup(test_hw_cap_l3ca_discover_non_contiguous_cbm,
+                                   _init),
             cmocka_unit_test_setup(test_hw_cap_l2ca_discover_alloc_unsupported,
                                    _init),
             cmocka_unit_test_setup(test_hw_cap_l2ca_discover_unsupported,
                                    _init),
             cmocka_unit_test_setup(test_hw_cap_l2ca_discover, _init),
             cmocka_unit_test_setup(test_hw_cap_l2ca_discover_cdp, _init),
+            cmocka_unit_test_setup(test_hw_cap_l2ca_discover_non_contiguous_cbm,
+                                   _init),
             cmocka_unit_test_setup(test_hw_cap_mba_discover_alloc_unsupported,
                                    _init),
             cmocka_unit_test_setup(test_hw_cap_mba_discover_unsupported, _init),
