@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,21 +30,13 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
+#include "hw_monitoring.h"
 #include "mock_cap.h"
 #include "mock_perf_monitoring.h"
-
-#include "hw_monitoring.h"
+#include "test.h"
 
 static int
-test_init_mon(void **state)
+wrap_init_mon(void **state)
 {
         int ret;
 
@@ -56,7 +48,7 @@ test_init_mon(void **state)
         if (ret == 0) {
                 struct test_data *data = (struct test_data *)*state;
 
-                ret = hw_mon_init(data->cpu, data->cap, NULL);
+                ret = hw_mon_init(data->cpu, data->cap);
                 assert_int_equal(ret, PQOS_RETVAL_OK);
         }
 
@@ -64,7 +56,7 @@ test_init_mon(void **state)
 }
 
 static int
-test_fini_mon(void **state)
+wrap_fini_mon(void **state)
 {
         int ret;
 
@@ -89,10 +81,16 @@ hw_mon_assoc_write(const unsigned lcore, const pqos_rmid_t rmid)
 
 int
 hw_mon_assoc_unused(struct pqos_mon_poll_ctx *ctx,
-                    const enum pqos_mon_event event)
+                    const enum pqos_mon_event event,
+                    pqos_rmid_t min_rmid,
+                    pqos_rmid_t max_rmid,
+                    const struct pqos_mon_options *opt)
 {
         assert_non_null(ctx);
         check_expected(event);
+        assert_int_equal(min_rmid, 1);
+        assert_int_equal(max_rmid, UINT32_MAX);
+        assert_non_null(opt);
 
         ctx->rmid = mock_type(int);
 
@@ -111,6 +109,9 @@ test_hw_mon_start_counter(void **state)
         struct pqos_mon_data_internal intl;
         enum pqos_mon_event event = PQOS_MON_EVENT_TMEM_BW;
         int ret;
+        struct pqos_mon_options opt;
+
+        memset(&opt, 0, sizeof(opt));
 
         memset(&group, 0, sizeof(struct pqos_mon_data));
         group.intl = &intl;
@@ -118,7 +119,8 @@ test_hw_mon_start_counter(void **state)
         group.cores = cores;
         memset(&intl, 0, sizeof(struct pqos_mon_data_internal));
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(hw_mon_assoc_unused, event, event);
         will_return(hw_mon_assoc_unused, 1);
@@ -128,7 +130,7 @@ test_hw_mon_start_counter(void **state)
         expect_value(hw_mon_assoc_write, rmid, 1);
         will_return(hw_mon_assoc_write, PQOS_RETVAL_OK);
 
-        ret = hw_mon_start_counter(&group, event);
+        ret = hw_mon_start_counter(&group, event, &opt);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(group.intl->hw.event, event);
         assert_int_equal(group.intl->hw.num_ctx, 1);
@@ -149,6 +151,9 @@ test_hw_mon_start_counter_core_group(void **state)
         struct pqos_mon_data_internal intl;
         enum pqos_mon_event event = PQOS_MON_EVENT_LMEM_BW;
         int ret;
+        struct pqos_mon_options opt;
+
+        memset(&opt, 0, sizeof(opt));
 
         memset(&group, 0, sizeof(struct pqos_mon_data));
         group.intl = &intl;
@@ -156,7 +161,8 @@ test_hw_mon_start_counter_core_group(void **state)
         group.cores = cores;
         memset(&intl, 0, sizeof(struct pqos_mon_data_internal));
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(hw_mon_assoc_unused, event, event);
         will_return(hw_mon_assoc_unused, 1);
@@ -174,7 +180,7 @@ test_hw_mon_start_counter_core_group(void **state)
         expect_value(hw_mon_assoc_write, rmid, 2);
         will_return(hw_mon_assoc_write, PQOS_RETVAL_OK);
 
-        ret = hw_mon_start_counter(&group, event);
+        ret = hw_mon_start_counter(&group, event, &opt);
         assert_int_equal(ret, PQOS_RETVAL_OK);
         assert_int_equal(group.intl->hw.event, event);
         assert_int_equal(group.intl->hw.num_ctx, 2);
@@ -197,7 +203,7 @@ main(void)
             cmocka_unit_test(test_hw_mon_start_counter),
             cmocka_unit_test(test_hw_mon_start_counter_core_group)};
 
-        result += cmocka_run_group_tests(tests, test_init_mon, test_fini_mon);
+        result += cmocka_run_group_tests(tests, wrap_init_mon, wrap_fini_mon);
 
         return result;
 }
