@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,23 +30,15 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <string.h>
-#include <stdlib.h>
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
-#include "test.h"
+#include "allocation.h"
 #include "mock_cap.h"
 #include "mock_common.h"
 #include "mock_resctrl.h"
 #include "mock_resctrl_alloc.h"
-#include "mock_resctrl_schemata.h"
 #include "mock_resctrl_monitoring.h"
-
-#include "allocation.h"
+#include "mock_resctrl_schemata.h"
 #include "os_allocation.h"
+#include "test.h"
 
 /* ======== mock ======== */
 
@@ -122,7 +114,8 @@ test_os_alloc_assoc_get(void **state)
         unsigned class_id = 0;
         unsigned lcore = 2;
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_shared, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -143,7 +136,8 @@ test_os_alloc_assoc_get_param(void **state)
         int ret;
         unsigned class_id;
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_alloc_assoc_get(1000, &class_id);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
@@ -161,7 +155,8 @@ test_os_alloc_assign(void **state)
         unsigned core_num = 2;
         unsigned core_array[] = {1, 2};
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_value(__wrap_resctrl_alloc_get_unused_group, grps_num, 3);
         will_return(__wrap_resctrl_alloc_get_unused_group, PQOS_RETVAL_OK);
@@ -190,7 +185,8 @@ test_os_alloc_release(void **state)
         unsigned core_num = 2;
         unsigned core_array[] = {1, 2};
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -216,7 +212,8 @@ test_os_alloc_release_param(void **state)
         unsigned core_num = 1;
         unsigned core_array[] = {1000};
 
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -237,8 +234,8 @@ test_os_l3ca_set_unsupported(void **state)
         int ret;
         struct pqos_l3ca ca[1];
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ca[0].class_id = 0;
         ca[0].cdp = 0;
@@ -255,8 +252,8 @@ test_os_l3ca_set_param(void **state)
         int ret;
         struct pqos_l3ca ca[1];
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ca[0].class_id = 0;
         ca[0].cdp = 0;
@@ -282,8 +279,8 @@ test_os_l3ca_set_cdp_off(void **state)
         ca[0].u.s.data_mask = 0xf0;
         ca[0].u.s.code_mask = 0xff;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -308,8 +305,8 @@ test_os_l3ca_set_cdp_on(void **state)
         ca[0].u.s.data_mask = 0xf0;
         ca[0].u.s.code_mask = 0xff;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -346,8 +343,8 @@ test_os_l3ca_set(void **state)
         ca[0].cdp = 0;
         ca[0].u.ways_mask = 0xf;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -381,8 +378,8 @@ test_os_l3ca_get_unsupported(void **state)
         unsigned l3cat_id = 0;
         unsigned num_ca = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_l3ca_get(l3cat_id, max_num_ca, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
@@ -398,15 +395,12 @@ test_os_l3ca_get_param(void **state)
         unsigned l3cat_id = 0;
         unsigned num_ca = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_l3ca_get(1000, max_num_ca, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = os_l3ca_get(l3cat_id, 1, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
@@ -422,8 +416,8 @@ test_os_l3ca_get(void **state)
         unsigned num_ca = 0;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_shared, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -452,8 +446,8 @@ test_os_l2ca_set_unsupported(void **state)
         int ret;
         struct pqos_l2ca ca[1];
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ca[0].class_id = 0;
         ca[0].cdp = 0;
@@ -470,8 +464,8 @@ test_os_l2ca_set_param(void **state)
         int ret;
         struct pqos_l2ca ca[1];
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ca[0].class_id = 0;
         ca[0].cdp = 0;
@@ -497,8 +491,8 @@ test_os_l2ca_set_cdp_off(void **state)
         ca[0].u.s.data_mask = 0xf0;
         ca[0].u.s.code_mask = 0xff;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -523,8 +517,8 @@ test_os_l2ca_set_cdp_on(void **state)
         ca[0].u.s.data_mask = 0xf0;
         ca[0].u.s.code_mask = 0xff;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -561,8 +555,8 @@ test_os_l2ca_set(void **state)
         ca[0].cdp = 0;
         ca[0].u.ways_mask = 0xf;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -596,8 +590,8 @@ test_os_l2ca_get_unsupported(void **state)
         unsigned l2cat_id = 0;
         unsigned num_ca = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_l2ca_get(l2cat_id, max_num_ca, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
@@ -613,15 +607,12 @@ test_os_l2ca_get_param(void **state)
         unsigned l2cat_id = 0;
         unsigned num_ca = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_l2ca_get(1000, max_num_ca, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = os_l2ca_get(l2cat_id, 1, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 }
@@ -637,8 +628,8 @@ test_os_l2ca_get(void **state)
         unsigned num_ca = 0;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_shared, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -667,8 +658,8 @@ test_os_mba_set_unsupported(void **state)
         int ret;
         struct pqos_mba ca[1];
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ca[0].class_id = 0;
         ca[0].ctrl = 0;
@@ -685,8 +676,8 @@ test_os_mba_set_param(void **state)
         int ret;
         struct pqos_mba ca[1];
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ca[0].class_id = 0;
         ca[0].ctrl = 0;
@@ -711,8 +702,8 @@ test_os_mba_set_ctrl_off(void **state)
         ca[0].ctrl = 1;
         ca[0].mb_max = 10000;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -736,8 +727,8 @@ test_os_mba_set_ctrl_on(void **state)
         ca[0].ctrl = 1;
         ca[0].mb_max = 10000;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -774,8 +765,8 @@ test_os_mba_set(void **state)
         ca[0].ctrl = 0;
         ca[0].mb_max = 60;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -809,8 +800,8 @@ test_os_mba_get_unsupported(void **state)
         unsigned mbat_id = 0;
         unsigned num_ca = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_mba_get(mbat_id, max_num_ca, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
@@ -826,15 +817,12 @@ test_os_mba_get_param(void **state)
         unsigned mbat_id = 0;
         unsigned num_ca = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_mba_get(1000, max_num_ca, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
-
         ret = os_mba_get(mbat_id, 1, &num_ca, ca);
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
@@ -850,8 +838,8 @@ test_os_mba_get(void **state)
         unsigned num_ca = 0;
         unsigned i;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-        will_return(__wrap__pqos_cap_get, data->cpu);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_shared, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -904,7 +892,8 @@ test_os_alloc_assign_pid(void **state)
         unsigned task_num = 2;
         pid_t task_array[] = {1, 2};
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -958,10 +947,10 @@ test_os_alloc_reset_unsupported_all(void **state)
         struct test_data *data = (struct test_data *)*state;
         int ret;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_ANY);
+        ret = os_alloc_reset(NULL);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 }
 
@@ -970,17 +959,23 @@ test_os_alloc_reset_unsupported_l3ca(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ON, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_ANY);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
+
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ON;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_OFF, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_ANY);
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_OFF;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 }
 
@@ -989,13 +984,21 @@ test_os_alloc_reset_unsupported_l3cdp(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
 
         data->cap_l3ca.cdp = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ON, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_ANY);
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ON;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 }
 
@@ -1004,17 +1007,23 @@ test_os_alloc_reset_unsupported_l2ca(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ON,
-                             PQOS_MBA_ANY);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
+
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ON;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_OFF,
-                             PQOS_MBA_ANY);
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_OFF;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 }
 
@@ -1023,13 +1032,21 @@ test_os_alloc_reset_unsupported_l2cdp(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
 
         data->cap_l2ca.cdp = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ON,
-                             PQOS_MBA_ANY);
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ON;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 }
 
@@ -1038,17 +1055,23 @@ test_os_alloc_reset_unsupported_mba(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_DEFAULT);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
+
+        cfg.mba = PQOS_MBA_DEFAULT;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_CTRL);
+        cfg.mba = PQOS_MBA_CTRL;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
 }
 
@@ -1057,13 +1080,22 @@ test_os_alloc_reset_unsupported_mba_ctrl(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
+        cfg.mba = PQOS_MBA_DEFAULT;
 
         data->cap_mba.ctrl = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_CTRL);
+        cfg.mba = PQOS_MBA_CTRL;
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_PARAM);
 }
 
@@ -1073,7 +1105,8 @@ test_os_alloc_reset_light(void **state)
         struct test_data *data = (struct test_data *)*state;
         int ret;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -1082,8 +1115,7 @@ test_os_alloc_reset_light(void **state)
         will_return(os_alloc_reset_schematas, PQOS_RETVAL_OK);
         will_return(os_alloc_reset_tasks, PQOS_RETVAL_OK);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_ANY);
+        ret = os_alloc_reset(NULL);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1094,8 +1126,6 @@ test_os_alloc_reset_prep(struct test_data *data)
 
         resctrl_alloc_get_grps_num(data->cap, &num_grps);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
-
         if (num_grps > 1) {
                 expect_string(__wrap_pqos_dir_exists, path,
                               "/sys/fs/resctrl/COS1");
@@ -1128,8 +1158,8 @@ test_os_alloc_reset_full(struct test_data *data,
                          enum pqos_cdp_config l2_cdp_cfg,
                          enum pqos_mba_config mba_cfg)
 {
-
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -1173,13 +1203,19 @@ test_os_alloc_reset_l3cdp_enable(void **state)
         enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ON;
         enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_mba_config mba_cfg = PQOS_MBA_ANY;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
 
         data->cap_l3ca.cdp = 1;
         data->cap_l3ca.cdp_on = 0;
 
         test_os_alloc_reset_full(data, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 
-        ret = os_alloc_reset(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1191,13 +1227,19 @@ test_os_alloc_reset_l3cdp_disable(void **state)
         enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_OFF;
         enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_mba_config mba_cfg = PQOS_MBA_ANY;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
 
         data->cap_l3ca.cdp = 1;
         data->cap_l3ca.cdp_on = 1;
 
         test_os_alloc_reset_full(data, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 
-        ret = os_alloc_reset(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1206,11 +1248,20 @@ test_os_alloc_reset_l3cdp_mon(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ON;
 
         data->cap_l3ca.cdp = 1;
         data->cap_l3ca.cdp_on = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -1219,8 +1270,7 @@ test_os_alloc_reset_l3cdp_mon(void **state)
         will_return(__wrap_resctrl_mon_active, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_mon_active, 1);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ON, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_ANY);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
 
@@ -1232,13 +1282,18 @@ test_os_alloc_reset_l2cdp_enable(void **state)
         enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ON;
         enum pqos_mba_config mba_cfg = PQOS_MBA_ANY;
+        struct pqos_alloc_config cfg;
 
+        memset(&cfg, 0, sizeof(cfg));
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
         data->cap_l2ca.cdp = 1;
         data->cap_l2ca.cdp_on = 0;
 
         test_os_alloc_reset_full(data, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 
-        ret = os_alloc_reset(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1250,13 +1305,19 @@ test_os_alloc_reset_l2cdp_disable(void **state)
         enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_OFF;
         enum pqos_mba_config mba_cfg = PQOS_MBA_ANY;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
 
         data->cap_l2ca.cdp = 1;
         data->cap_l2ca.cdp_on = 1;
 
         test_os_alloc_reset_full(data, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 
-        ret = os_alloc_reset(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1265,11 +1326,20 @@ test_os_alloc_reset_l2cdp_mon(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ON;
 
         data->cap_l2ca.cdp = 1;
         data->cap_l2ca.cdp_on = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -1278,8 +1348,7 @@ test_os_alloc_reset_l2cdp_mon(void **state)
         will_return(__wrap_resctrl_mon_active, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_mon_active, 1);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ON,
-                             PQOS_MBA_ANY);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
 
@@ -1291,13 +1360,19 @@ test_os_alloc_reset_mba_ctrl_enable(void **state)
         enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_mba_config mba_cfg = PQOS_MBA_CTRL;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
 
         data->cap_mba.ctrl = 1;
         data->cap_mba.ctrl_on = 0;
 
         test_os_alloc_reset_full(data, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 
-        ret = os_alloc_reset(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1309,13 +1384,19 @@ test_os_alloc_reset_mba_ctrl_disable(void **state)
         enum pqos_cdp_config l3_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_cdp_config l2_cdp_cfg = PQOS_REQUIRE_CDP_ANY;
         enum pqos_mba_config mba_cfg = PQOS_MBA_DEFAULT;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+        cfg.l3_cdp = l3_cdp_cfg;
+        cfg.l2_cdp = l2_cdp_cfg;
+        cfg.mba = mba_cfg;
 
         data->cap_mba.ctrl = 1;
         data->cap_mba.ctrl_on = 1;
 
         test_os_alloc_reset_full(data, l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
 
-        ret = os_alloc_reset(l3_cdp_cfg, l2_cdp_cfg, mba_cfg);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_OK);
 }
 
@@ -1324,11 +1405,20 @@ test_os_alloc_reset_mba_ctrl_mon(void **state)
 {
         struct test_data *data = (struct test_data *)*state;
         int ret;
+        struct pqos_alloc_config cfg;
+
+        memset(&cfg, 0, sizeof(cfg));
+#if PQOS_VERSION < 50000
+        cfg.l3_cdp = PQOS_REQUIRE_CDP_ANY;
+        cfg.l2_cdp = PQOS_REQUIRE_CDP_ANY;
+#endif
+        cfg.mba = PQOS_MBA_CTRL;
 
         data->cap_mba.ctrl = 1;
         data->cap_mba.ctrl_on = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         will_return(__wrap_resctrl_lock_exclusive, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_lock_release, PQOS_RETVAL_OK);
@@ -1337,8 +1427,7 @@ test_os_alloc_reset_mba_ctrl_mon(void **state)
         will_return(__wrap_resctrl_mon_active, PQOS_RETVAL_OK);
         will_return(__wrap_resctrl_mon_active, 1);
 
-        ret = os_alloc_reset(PQOS_REQUIRE_CDP_ANY, PQOS_REQUIRE_CDP_ANY,
-                             PQOS_MBA_CTRL);
+        ret = os_alloc_reset(&cfg);
         assert_int_equal(ret, PQOS_RETVAL_ERROR);
 }
 
@@ -1352,7 +1441,8 @@ test_os_alloc_init_mounted(void **state)
         expect_string(__wrap_pqos_file_exists, path, "/sys/fs/resctrl/cpus");
         will_return(__wrap_pqos_file_exists, 1);
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         resctrl_alloc_get_grps_num(data->cap, &num_grps);
 
@@ -1392,6 +1482,9 @@ test_os_alloc_init_unmounted(void **state)
         expect_string(__wrap_pqos_file_exists, path, "/sys/fs/resctrl/cpus");
         will_return(__wrap_pqos_file_exists, 0);
 
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
+
         expect_value(os_alloc_mount, l3_cdp_cfg, PQOS_REQUIRE_CDP_OFF);
         expect_value(os_alloc_mount, l2_cdp_cfg, PQOS_REQUIRE_CDP_OFF);
         expect_value(os_alloc_mount, mba_cfg, PQOS_MBA_DEFAULT);
@@ -1419,7 +1512,8 @@ test_os_l3ca_get_min_cbm_bits(void **state)
         int ret;
         unsigned min_cbm_bits = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_string(__wrap_pqos_fread_uint64, fname,
                       "/sys/fs/resctrl/info/L3/min_cbm_bits");
@@ -1439,7 +1533,8 @@ test_os_l3ca_get_min_cbm_bits_unsupported(void **state)
         int ret;
         unsigned min_cbm_bits;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_l3ca_get_min_cbm_bits(&min_cbm_bits);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
@@ -1454,7 +1549,8 @@ test_os_l2ca_get_min_cbm_bits(void **state)
         int ret;
         unsigned min_cbm_bits = 0;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         expect_string(__wrap_pqos_fread_uint64, fname,
                       "/sys/fs/resctrl/info/L2/min_cbm_bits");
@@ -1474,7 +1570,8 @@ test_os_l2ca_get_min_cbm_bits_unsupported(void **state)
         int ret;
         unsigned min_cbm_bits;
 
-        will_return(__wrap__pqos_cap_get, data->cap);
+        will_return_maybe(__wrap__pqos_get_cap, data->cap);
+        will_return_maybe(__wrap__pqos_get_cpu, data->cpu);
 
         ret = os_l2ca_get_min_cbm_bits(&min_cbm_bits);
         assert_int_equal(ret, PQOS_RETVAL_RESOURCE);
@@ -1566,4 +1663,4 @@ main(void)
                                          test_init_unsupported, test_fini);
 
         return result;
-}
\ No newline at end of file
+}
