@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2022 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -42,18 +42,6 @@ __wrap__pqos_check_init(const int expect)
         return mock_type(int);
 }
 
-void
-__wrap__pqos_api_lock(void)
-{
-        function_called();
-}
-
-void
-__wrap__pqos_api_unlock(void)
-{
-        function_called();
-}
-
 const struct pqos_cap *
 __wrap__pqos_get_cap(void)
 {
@@ -66,6 +54,12 @@ __wrap__pqos_get_cpu(void)
         return mock_ptr_type(const struct pqos_cpuinfo *);
 }
 
+enum pqos_interface
+__wrap__pqos_get_inter(void)
+{
+        return mock_type(enum pqos_interface);
+}
+
 void
 __wrap__pqos_cap_l3cdp_change(const enum pqos_cdp_config cdp)
 {
