@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,13 +30,10 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
-
 #include "mock_hw_allocation.h"
 
+#include "mock_test.h"
+
 int
 __wrap_hw_alloc_assoc_set(const unsigned lcore, const unsigned class_id)
 {
@@ -91,13 +88,9 @@ __wrap_hw_alloc_release(const unsigned *core_array, const unsigned core_num)
 }
 
 int
-__wrap_hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-                      const enum pqos_cdp_config l2_cdp_cfg,
-                      const enum pqos_mba_config mba_cfg)
+__wrap_hw_alloc_reset(const struct pqos_alloc_config *cfg)
 {
-        check_expected(l3_cdp_cfg);
-        check_expected(l2_cdp_cfg);
-        check_expected(mba_cfg);
+        check_expected_ptr(cfg);
 
         return mock_type(int);
 }
@@ -214,4 +207,4 @@ __wrap_hw_alloc_assoc_read(const unsigned lcore, unsigned *class_id)
         *class_id = mock_type(int);
 
         return mock_type(int);
-}
\ No newline at end of file
+}
