@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -43,9 +43,7 @@ int __wrap_hw_alloc_assign(const unsigned technology,
                            unsigned *class_id);
 int __wrap_hw_alloc_release(const unsigned *core_array,
                             const unsigned core_num);
-int __wrap_hw_alloc_reset(const enum pqos_cdp_config l3_cdp_cfg,
-                          const enum pqos_cdp_config l2_cdp_cfg,
-                          const enum pqos_mba_config mba_cfg);
+int __wrap_hw_alloc_reset(const struct pqos_alloc_config *cfg);
 int __wrap_hw_l3ca_set(const unsigned l3cat_id,
                        const unsigned num_cos,
                        const struct pqos_l3ca *ca);
