@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -37,13 +37,19 @@
 
 int __wrap_hw_mon_reset(void);
 int __wrap_hw_mon_assoc_get(const unsigned lcore, pqos_rmid_t *rmid);
-int __wrap_hw_mon_start(const unsigned num_cores,
-                        const unsigned *cores,
-                        const enum pqos_mon_event event,
-                        void *context,
-                        struct pqos_mon_data *group);
+int __wrap_hw_mon_start_cores(const unsigned num_cores,
+                              const unsigned *cores,
+                              const enum pqos_mon_event event,
+                              void *context,
+                              struct pqos_mon_data *group,
+                              const struct pqos_mon_options *opt);
 int __wrap_hw_mon_stop(struct pqos_mon_data *group);
 int __wrap_hw_mon_poll(struct pqos_mon_data *group,
                        const enum pqos_mon_event event);
+int __wrap_hw_mon_start_uncore(const unsigned num_sockets,
+                               const unsigned *sockets,
+                               const enum pqos_mon_event event,
+                               void *context,
+                               struct pqos_mon_data **group);
 
 #endif /* MOCK_HW_MONITORING_H_ */
