@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,12 +30,10 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include <stdarg.h>
-#include <stddef.h>
-#include <setjmp.h>
-#include <cmocka.h>
 #include "mock_os_monitoring.h"
 
+#include "mock_test.h"
+
 int
 __wrap_os_mon_reset(void)
 {
@@ -43,17 +41,19 @@ __wrap_os_mon_reset(void)
 }
 
 int
-__wrap_os_mon_start(const unsigned num_cores,
-                    const unsigned *cores,
-                    const enum pqos_mon_event event,
-                    void *context,
-                    struct pqos_mon_data *group)
+__wrap_os_mon_start_cores(const unsigned num_cores,
+                          const unsigned *cores,
+                          const enum pqos_mon_event event,
+                          void *context,
+                          struct pqos_mon_data *group,
+                          const struct pqos_mon_options *opt)
 {
         check_expected(num_cores);
         check_expected_ptr(cores);
         check_expected(event);
         check_expected_ptr(context);
-        check_expected_ptr(group);
+        assert_non_null(group);
+        assert_non_null(opt);
 
         return mock_type(int);
 }
@@ -86,7 +86,7 @@ __wrap_os_mon_start_pids(const unsigned num_pids,
         assert_non_null(pids);
         check_expected(event);
         check_expected_ptr(context);
-        check_expected_ptr(group);
+        assert_non_null(group);
 
         return mock_type(int);
 }
