@@ -1,7 +1,7 @@
 /*
  * BSD LICENSE
  *
- * Copyright(c) 2020-2021 Intel Corporation. All rights reserved.
+ * Copyright(c) 2020-2023 Intel Corporation. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,11 +36,12 @@
 #include "os_monitoring.h"
 
 int __wrap_os_mon_reset(void);
-int __wrap_os_mon_start(const unsigned num_cores,
-                        const unsigned *cores,
-                        const enum pqos_mon_event event,
-                        void *context,
-                        struct pqos_mon_data *group);
+int __wrap_os_mon_start_cores(const unsigned num_cores,
+                              const unsigned *cores,
+                              const enum pqos_mon_event event,
+                              void *context,
+                              struct pqos_mon_data *group,
+                              const struct pqos_mon_options *opt);
 int __wrap_os_mon_stop(struct pqos_mon_data *group);
 int __wrap_os_mon_poll(struct pqos_mon_data *group,
                        const enum pqos_mon_event event);
@@ -56,4 +57,9 @@ int __wrap_os_mon_remove_pids(const unsigned num_pids,
                               const pid_t *pids,
                               struct pqos_mon_data *group);
 
+/* ======== headers for static functions ======== */
+int os_mon_stop_events(struct pqos_mon_data *group);
+int os_mon_start_events(struct pqos_mon_data *group);
+int os_mon_tid_exists(const pid_t pid);
+
 #endif /* MOCK_OS_MONITORING_H_ */
